unit Nwcalls;

{
 The following declarations refer to the DLLs that come with the NetWare client for Windows.
 The current versions of these DLLs can be found in the area NetWire/NOVFILES on
 CompuServe or ftp.novell.com

 This is just a subset of all the available functions. A complete description can be found
 in the NetWare Client SDK.

 To get the complete NetWare Client SDK contact
    1-800-REDWORD    in the US,
    +61-2565-6098    in Asia,
    +31-55-384279    in Europe


 The type and constant definitions refer to the matching C declarations in the Client SDK,
 to allow easier conversion from C syntax into Pascal
 }


interface

uses    WinCrt,
        Controls, Dialogs, Forms, SysUtils, StdCtrls,
        Delphi_u;

CONST
   Success         = 0000;

   Warning         = 1;
   Err             = 2;
   Fatal           = 3;

{*** Bindery security definitions ***}
	BS_ANY_READ		=$00; {Readable by anyone}
	BS_LOGGED_READ		=$01; {Must be logged in to read}
	BS_OBJECT_READ		=$02; {Readable by same object or super}
	BS_SUPER_READ		=$03; {Readable by supervisor only}
	BS_BINDERY_READ	        =$04; {Readable only by the bindery}
	BS_ANY_WRITE		=$00; {Writeable by anyone}
	BS_LOGGED_WRITE	        =$10; {Must be logged in to write}
	BS_OBJECT_WRITE	        =$20; {Writeable by same object or super}
	BS_SUPER_WRITE		=$30; {Writeable only by the supervisor}
	BS_BINDERY_WRITE	=$40; {Writeable by the bindery only}

{Bindery object type definitions}
	OT_WILD			=$FFFF;{Matches any type}
	OT_UNKNOWN		=$00;	{Unknown object type}
	OT_USER			=$01;	{The object is a "user"}
	OT_USER_GROUP		=$02;	{A group of users}
	OT_PRINT_QUEUE		=$03;	{Services print queues}
	OT_FILE_SERVER		=$04;	{The object serves files}
	OT_JOB_SERVER		=$05;
	OT_GATEWAY		=$06;
	OT_PRINT_SERVER		=$07;
	OT_ARCHIVE_QUEUE	=$08;
	OT_ARCHIVE_SERVER	=$09;	{Services backup jobs}
	OT_JOB_QUEUE		=$0A;
	OT_ADMINISTRATION	=$0B;
	OT_NAS_SNA_GATEWAY	=$21;
	OT_NACS		        =$23;
	OT_REMOTE_BRIDGE_SERVER	=$24;
	OT_REMOTE_BRIDGE_SERVER_2	=$26	{38 CONFLICT EXISTS!!};
 	OT_TCPIP_GATEWAY		=$27; {39 CONFLICT EXISTS!!}
 	OT_GATEWAY2			=$29; {39 CONFLICT EXISTS!!}
	OT_TIME_SYNCHRONIZATION_SERVER  =$2D;
	OT_ARCHIVE_SERVER_DYNAMIC_SAP	=$2E;
	OT_ADVERTISING_PRINT_SERVER	=$47;
	OT_BTRIEVE_VAP			=$4B;	{prior to 2.15, 50}
	OT_SQL_VAP_NLM                  =$4C;
        OT_XTREE_NET                    =$4D;
	OT_BTRIEVE_VAP_4xx              =$50;	{prior to 2.15, 50}
	OT_PRINT_QUEUE_USER		=$53; {AppleTalk}
	OT_X25_Gateway                  =$58; { 88}
	OT_WANCopy                      =$72; {114}
	OT_TES_NetWare_for_VMS          =$7A; {122}
	OT_NetWare_Access_Server       =$0098; {152}
	OT_Portable_NetWare            =$009E; {158}
	OT_Compaq_IDA_Status_Monitor   =$00AC; {172}
	OT_NetWare_386_Server          =$0107; {263}
	OT_CSA_MUX                     =$0114; {276}
	OT_CSA_LCA                     =$0115; {277}
	OT_CSA_CM                      =$0116; {278}
	OT_CSA_SMA                     =$0117; {279}
	OT_CSA_DBA                     =$0118; {280}
	OT_CSA_NMA                     =$0119; {281}
	OT_CSA_SSA                     =$011A; {282}
	OT_CSA_STATUS                  =$011B; {283}
	OT_CSA_APPC                    =$011E; {286}
	OT_SNA_TEST                    =$0126; {294}
	OT_CSA_TRACE                   =$012A; {298}
	OT_Communications_Executive    =$0130; {304}
	OT_NNS_Domain                  =$0133; {307}
	OT_NNS_Profile                 =$0135; {309}
	OT_NNS_Queue                   =$0137; {311}
        OT_NDS_TREE                    =$0278;
	OT_Wordperfect_Network_Version =$8888;

{Attributes of objects and properties in the bindery}
	BF_STATIC		         =$00;
	BF_DYNAMIC		         =$01;
	BF_ITEM			         =$00;
	BF_SET			         =$02;

	RI_UNDEFINED		         =	-1;
	RI_GETCONNECTIONSOPENFILES	 =	1;
	RI_GETCONNECTIONSSEMAPHORES	 =	2;
	RI_GETCONNECTIONSTASKINFO	 =	3;
	RI_GETCURRENTACCOUNTSTATUS	 =	4;
	RI_GETLOGICALRECORDINFORMATION	 =	5;
	RI_GETLOGICALRECORDSBYCONNECTIO	 =	6;
	RI_GETPHYSICALRECORDLOCKSBYFILE	 =	7;
	RI_GETPHYSRECLOCKBYCONNECTANDFI	 =	8;
	RI_GETSEMAPHOREINFORMATION	=	9;
	RI_TTSGETSTATS			=	10;
	RI_GETCONNECTIONSUSINGFILE	=	11;

{Maximum size of NetWare buffers in bytes}
	MAX_NETWARE_BUFFER_SIZE		=	512;
        FSE_MAX_NUM_OF_LANS             =       16{32};

{File Attributes}
	FA_NORMAL			= $00;
	FA_READ_ONLY			= $01;
	FA_HIDDEN			= $02;
	FA_SYSTEM			= $04;
	FA_EXECUTE_ONLY			= $08;
	FA_DIRECTORY			= $10;
	FA_NEEDS_ARCHIVED		= $20;
	FA_SHAREABLE			= $80;
	FA_ALLFILES			= $AF;

{Extended file attributes}
	FA_TRANSACTIONAL		= $10;
	FA_INDEXED			= $20;
	FA_READ_AUDIT			= $40;
	FA_WRITE_AUDIT			= $80;

{File Lock Status}
	FL_LOCKED			=$01;
	FL_OPEN_SHARABLE	        =$02;
	FL_LOGGED			=$04;
	FL_OPEN_NORMAL		        =$08;
	FL_TTS_LOCK			=$40;
	FL_TTS_FLAG_SET	                =$80;

{Trustee Access Rights in a network directory (286)}
        TA_NONE				=$00;
	TA_READ				=$01;
	TA_WRITE			=$02;
	TA_OPEN				=$04;
	TA_CREATE			=$08;
	TA_DELETE			=$10;
	TA_OWNERSHIP		        =$20;
	TA_ACCESS		        =$20;
	TA_SEARCH			=$40;
	TA_MODIFY			=$80;
	TA_ALL				=$FF;

	TR_NONE				 =$0000; {386 TRUSTEE RIGHTS}
	TR_READ				 =$0001;
	TR_WRITE			 =$0002;
	TR_CREATE			 =$0008;
	TR_ERASE			 =$0010;
	TR_ACCESS			 =$0020;
	TR_FILE				 =$0040;
	TR_MODIFY			 =$0080;
	TR_SUPERVISOR			 =$0100;
	TR_NORMAL			 =$00FF;
	TR_ALL				 =$01FF;

{ADDITIONAL EXTENDED FILE ATTRIBUTES}
	TF_NORMAL			 =$0000;
	TF_READ_ONLY			 =$0001;
	TF_HIDDEN			 =$0002;
	TF_SYSTEM			 =$0004;
	TF_EXECUTE_ONLY			 =$0008;

	TF_DIRECTORY			 =$0010;
	TF_NEEDS_ARCHIVED		 =$0020;
	TF_EXECUTE_CONFIRM		 =$0040;
	TF_SHAREABLE			 =$0080;

	TF_LOW_SEARCH			 =$0100;
	TF_MID_SEARCH			 =$0200;
	TF_HI_SEARCH			 =$0400;
	TF_PRIVATE			 =$0800;

	TF_TRANSACTIONAL		 =$1000;
	TF_INDEXED			 =$2000;
	TF_READ_AUDIT			 =$4000;
	TF_WRITE_AUDIT			 =$8000;

	TF_PURGE			 =$10000;
	TF_RENAME_INHIBIT		 =$20000;
	TF_DELETE_INHIBIT		 =$40000;
	TF_COPY_INHIBIT			 =$80000;


{DIRECTORY ATTRIBUTES}
	TD_HIDDEN			 = TF_HIDDEN;
	TD_SYSTEM			 = TF_SYSTEM;
	TD_PURGE			 = TF_PURGE;
	TD_PRIVATE			 = TF_PRIVATE;
	TD_VISIBLE			 = TF_PRIVATE;
	TD_RENAME_INHIBIT		 = TF_RENAME_INHIBIT;
	TD_DELETE_INHIBIT		 = TF_DELETE_INHIBIT;

        SESSION_KEY_SIZE                 = 16;

        SYSTEM_LOCK                      = 0;
        TASK_LOCK                        = 4;
        SYSTEM_DISCONNECT                = 0;
        TASK_DISCONNECT                  = 1;

        OpenRead                         = $0;
        OpenWrite                        = $1;
        OpenRdWr                         = $2;
        DenyRdWr                         = $10;
        DenyWrite                        = $20;
        DenyNone                         = $40;

        LC_ALL                           = 0;  { used with NWLsetlocale }
        LC_COLLATE                       = 1;
        LC_CTYPE                         = 2;
        LC_MONETARY                      = 3;
        LC_NUMERIC                       = 4;
        LC_TIME                          = 5;

        MAX_DN_CHARS                     = 256;

        ALREADY_ATTACHED                 = $8800;     {  Error Codes }


TYPE
     Str2          = String[2];
     Str4          = String[4];
     Str8          = String[8];
     Str16         = String[16];
     Str24         = String[24];
     Str32         = String[32];
     Str48         = String[48];

     uint8                     = Byte;
     Int16                     = Word;
     int                       = LongInt;
     int4                      = LongInt;
     uint32                    = LongInt;
     DWORD                     = LongInt;

     NWCCODE                   = WORD;
     NWCONN_HANDLE             = WORD;
     NWCHANGE_BITS             = DWORD;
     NWCONN_NUM                = WORD;
     NWCONN_NUM_WORD           = WORD;
     NWCONN_TYPE               = BYTE;
     NWSTATUS                  = DWORD;
     NWOBJ_TYPE                = WORD;
     NWOBJ_ID                  = DWORD;
     NWSIZE                    = DWORD;
     NWLEN                     = DWORD;
     NWACCT_BALANCE            = DWORD;
     NWACCT_LIMIT              = DWORD;
     NWACCT_HOLDS              = WORD;
     NWCHARGE_AMOUNT           = DWORD;
     NWHOLD_CANCEL_AMOUNT      = DWORD;
     NWHOLD_AMOUNT             = DWORD;
     NWNOTE_TYPE               = WORD;
     NWPSTR                    = PChar;
     NWAFP_ENTRY_ID            = DWORD;
     NWAFP_FILE_ATTRIBUTES     = WORD;
     NWAFP_FORK_LEN            = DWORD;
     NWAFP_NUM_OFFSPRING       = WORD;
     NWDATE                    = WORD;
     NWTIME                    = WORD;
     NWFINDER_INFO             = BYTE;
     NWAFP_ACCESS_PRIVILEGES   = WORD;
     NWDIR_TRUSTEE_RIGHTS      = WORD;
     NWACCESS_RIGHTS           = BYTE;
     TRUSTEE_RIGHTS            = DWORD;
     NWVOL_NUM                 = WORD;
     NWFLAGS                   = BYTE;
     NWHANDLE                  = BYTE;
     NWREQUEST_MASK            = WORD;
     NWSTRUCT_SIZE             = WORD;
     NWFILE_MODE               = BYTE;
     NWLOCAL_FILE_HANDLE       = WORD;
     NWSEARCH_MASK             = WORD;
     NWADDR_TYPE               = BYTE;
     NWADDR_LEN                = BYTE;
     NWCOUNT                   = DWORD;
     NWMINUTES                 = BYTE;
     NWSECONDS                 = DWORD;
     NWDATE_TIME               = DWORD;
     NWATTR                    = DWORD;
     FILE_ATTRIBUTES_MASK      = DWORD;
     NWRIGHTS_MASK             = WORD;
     NWDISK_SPACE              = DWORD;
     NWNUM_BYTES               = DWORD;
     NWOFFSET                  = DWORD;
     NWNUM_BLOCKS              = DWORD;
     NWSEGMENT_NUM             = BYTE;
     NWSEGMENT_DATA            = PChar;
     NWSEQUENCE                = DWORD;
     NWSYS_TIME                = DWORD;
     NWTRUSTEE_SEQUENCE_NUM    = WORD;
     NWELEMENT_VALUE           = INT16;
     NWNUM_ELEMENTS            = INT16;
     NWCONFIG_ELEMENT_NUM      = INT16;
     NWCONFIG_PARAM_TYPE       = INT16;
     NWCONFIG_DEFAULT_VALUE    = DWORD;
     NWNUMBER                  = WORD;
     NWNUM                     = DWORD;
     NWCONN_FLAGS              = WORD;
     NWSESSION_ID              = WORD;
     NWNUM_PACKETS             = DWORD;
     NWLOGIN_TIME              = Array[1..7] of BYTE;
     NWLOCAL_MODE              = WORD;
     NWLOCAL_SCOPE             = WORD;
     NWNAME_SPACE              = BYTE;
     NWDIR_ID                  = BYTE;
     NWDIR_NUM                 = WORD;
     NWDIR_ENTRY               = DWORD;
     NWNAME                    = PChar;
     NWNAME_LEN                = BYTE;
     NWFILE_LEN                = DWORD;
     NWDELETE_TIME             = DWORD;
     NWVOL                     = DWORD;
     NWDIR_SPACE               = DWORD;
     NWFAT                     = DWORD;
     NWSEARCH_ATTRIBUTES       = WORD;
     NWSEARCH_ATTR             = BYTE;
     NWAUGMENT                 = WORD;
     NWNUM_ENTRIES             = WORD;
     NWBITS                    = DWORD;
     NWNUM_DIR_ENTRIES         = DWORD;
     NWRESERVED32              = DWORD;
     NWRESERVED16              = DWORD;
     NWNUM_SECTORS             = BYTE;
     NWSEQUENCE_NUM            = WORD;
     NWDIR_ATTRIBUTES          = BYTE;
     NWDIR_STAMP               = WORD;
     NWDRIVE_NUM               = WORD;
     NWPATH_VOL                = BYTE;
     NWDIR_VOL                 = BYTE;
     NWNET_ADDR_TYPE           = DWORD;
     NWNET_ADDR_LEN            = DWORD;
     NWNET_ADDR                = BYTE;
     NWPOSITION                = DWORD;
     NWEA_HANDLE               = DWORD;
     NWEA_KEY_LEN              = WORD;
     NWEA_KEY_OFFSET           = WORD;
     NWEA_KEY                  = WORD;
     NWFILE_ATTR               = BYTE;
     NWVCONSOLE_VERSION        = BYTE;
     NWVCONSOLE_REVISION       = BYTE;
     NWFACTOR                  = DWORD;
     NWNUM_HOPS                = WORD;
     NWNUM_BUFFERS             = WORD;
     NWUTILIZATION             = DWORD;
     NWLOCK_COUNT              = WORD;
     NWNUM_REQUESTS            = DWORD;
     NWSOCKET_COUNT            = WORD;
     NWAES_COUNT               = WORD;
     NWECB_CANCEL_COUNT        = WORD;
     NWGLT_FAIL_COUNT          = WORD;
     NWSPX_COUNT               = WORD;
     NWFSE_FLAGS               = DWORD;
     NWNLM_TYPE                = DWORD;
     NWNLM_ID                  = DWORD;
     NWOS_VERSION              = WORD;
     NWOS_REVISION             = WORD;
     NWAPP_NUM                 = WORD;
     NWDRV_FLAGS               = WORD;
     NWDRV_NAME                = DWORD;
     NWDRV_TYPE                = DWORD;
     NWDRV_ID                  = WORD;
     NWDRV_VERSION             = BYTE;
     NWDRV_LINK                = DWORD;
     NWDRV_MEM                 = DWORD;
     NWDRV_TAG                 = DWORD;
     NWDRV_CONFIG              = DWORD;
     NWDRV_COMMAND             = DWORD;
     NWSTATS_VERSION           = BYTE;
     NWMEDIA_TYPE              = DWORD;
     NWMEDIA_MASK              = DWORD;
     NWPROTOCOL_VERSION        = BYTE;
     NWPROTOCOL_MASK           = DWORD;
     NWFSE_CONN_TYPE           = DWORD;
     NWVERSION                 = DWORD;
     NWREVISION                = DWORD;
     NWFILE_SYS_ID             = DWORD;
     NWTRAN_TYPE               = BYTE;
     NWLENGTH                  = WORD;
     NWBUF_SIZE                = WORD;
     NWDISTANCE                = WORD;
     NWMAX_PACKET_SIZE         = WORD;
     NWSERVER_NAME_LEN         = WORD;
     NWRIGHTS                  = DWORD;
     NWTASK_NUM                = WORD;
     NWLOCK_TYPE               = BYTE;
     NWNUM_CONNS               = BYTE;
     NWNEXT_REQUEST            = WORD;
     NWTASK                    = WORD;
     NWNUM_FORKS               = BYTE;
     NWOPEN_COUNT              = WORD;
     NWCURRENT_REC             = WORD;
     NWUSE_COUNT               = WORD;
     NWDENY_COUNT              = WORD;
     NWSEARCH_CONTEXT          = WORD;
     NWSEQ_NUM                 = BYTE;
     NWLAST_RECORD             = INT16;
     NWTICKS                   = WORD;
     NWMODULE_ID               = DWORD;
     NWDATA_STREAM             = DWORD;
     NWINFO_LEVEL              = DWORD;
     NWDM_FLAGS                = DWORD;
     NWFRAG_SIZE               = WORD;
     NWTASK_STATE              = BYTE;
     NWLOCK_STATE              = BYTE;
     NWTASK_COUNT              = BYTE;
     NWREQUESTER_VERSION       = BYTE;
     NWACCESS_MODE             = BYTE;
     NWBROADCAST_MODE          = WORD;
     NWDIR_BASE                = DWORD;
     NWNS_MASK                 = DWORD;
     NWNS_BITS                 = WORD;
     NWNS_FLAGS                = WORD;
     NWPATH_SIZE               = WORD;
     NWNS_ACCESS_RIGHTS        = WORD;
     NWNS_HANDLE               = DWORD;
     NWNS_ATTR                 = WORD;
     NWNS_LIST_SIZE            = BYTE;
     NWNS_NUM                  = BYTE;
     NWNS_DATA_STREAM          = BYTE;
     NWNS_TYPE                 = WORD;
     NWNS_DATA_STREAM2         = WORD;
     NWNS_ACCESS_MODE          = WORD;
     NWFORM_TYPE               = WORD;
     NWSTATION_NUM             = BYTE;
     NWSTATION_NUM2            = DWORD;
     NWJOB_POSITION            = BYTE;
     NWJOB_POSITION2           = WORD;
     NWTAB                     = BYTE;
     NWPRINT_TASK              = DWORD;
     NWPRINT_FLAGS             = WORD;
     NWJOB_TYPE                = WORD;
     NWQMS_TASK                = DWORD;
     NWJOB_FLAGS               = WORD;
     NWQMS_HANDLE              = DWORD;
     NWJOB_HANDLE              = DWORD;
     NWNUMBER_ENTRIES          = BYTE;
     NWDRIVE_NUMBER            = BYTE;
     NWDIR_HANDLE              = BYTE;
     NWVOL_TYPE                = DWORD;
     NWVOL_NUMBER              = BYTE;
     NWVOL_FLAGS               = WORD;
     NWNUM_TRANSACTIONS        = BYTE;
     NWTDS                     = WORD;
     NWTDS_OFFSET              = WORD;
     NWREC_OFFSET              = WORD;
     NWSEM_VALUE               = WORD;
     NWLOCK_DATA_STREAM        = BYTE;
     NWSEM_HANDLE              = DWORD;
     NWSEM_INT                 = INT16;
     NWOPTION_NUM              = BYTE;
     NWDISK_CHANNEL            = BYTE;
     NWDISK_TYPE               = BYTE;
     NWCTLR_NUM                = BYTE;
     NWCTLR_TYPE               = BYTE;
     NWNUM_HEADS               = BYTE;
     NWHF_START                = DWORD;
     NWDISK_FLAGS              = WORD;
     NWDISK_DRV_TYPE           = BYTE;
     NWIO_MEM                  = WORD;
     NWMEM_OFFSET              = WORD;
     NWINTERRUPT               = BYTE;
     NWDMA                     = BYTE;
     NWSUPPORT_LEVEL           = BYTE;
     NWNUM_DRIVES              = BYTE;
     NWSERVICE_VERSION         = BYTE;
     NWDISK_NUM                = BYTE;
     NWLAN_NUM                 = BYTE;
     NWSERVER_TYPE             = WORD;
     NWPRINTER                 = WORD;
     NWNUM_COPIES              = BYTE;
     NWFORM_NUM                = BYTE;
     NWDEVICE_ID               = WORD;
     NWLPT                     = BYTE;
     NWREPLICA_NUM             = DWORD;

     NWAPI                     = Pointer;
     NWDSCCODE                 = int;
     NWDS_FLAGS                = uint32;
     NWDS_Session_Key_T        = Array[1..SESSION_KEY_SIZE] of uint8;   { Optional session key }
     NWDS_Session_Key          = NWDS_Session_Key_T;
     PNWDS_Session_Key         = ^NWDS_Session_Key;

     NWSERVER_ADDR             = Array[1..12]  of Byte;
     NWPath                    = Array[1..255] of Char;

TYPE TPropSegm                 = Array[1..128] of Byte;
     TIDList                   = Array[1..32]  of LongInt;
     TConnList                 = Array[1..50]  of Word;
     NWPropertyName            = Array[1..16]  of Char;
     NWObjectName              = Array[1..48]  of Char;
     NDSName                   = Array[1..MAX_DN_CHARS] of Char;
     TNWConnHandleList         = Array[0..19]  of NWCONN_HANDLE;

TYPE TAddress = Record
     network:      Array[1..4] of BYTE;
     node:         Array[1..6] of BYTE;
     end;

TYPE scanObjectRep = RECORD
     ObjectID	         : NWOBJ_ID;
     ObjectType	         : NWOBJ_TYPE;
     ObjectName          : NWObjectName;
     ObjectFlag	         : NWFLAGS;
     ObjectSecurity      : NWFLAGS;
     ObjectHasProperties : NWFLAGS;
     END;

TYPE scanPropRep = RECORD
     sequenceNumber:    NWSEQUENCE;
     propertyName:      NWPropertyName;
     propertyFlags:     NWFLAGS;
     propertySecurity:  NWFLAGS;
     hasValueFlag:      NWFLAGS;
     moreFlag:          NWFLAGS;
     END;

TYPE readPropValueRep = RECORD
     dataSetIndex:      NWSEGMENT_NUM;
     data:              TPropSegm;
     moreFlag:          NWFLAGS;
     propertyFlags:     NWFLAGS;
     END;

TYPE CONNECT_INFO =     RECORD
     connID:            NWCONN_HANDLE;
     connectFlags:      NWCONN_FLAGS;
     sessionID:         NWSESSION_ID;
     connNumber:        NWCONN_NUM;
     serverAddr:        NWSERVER_ADDR;
     serverType:        NWOBJ_TYPE;
     serverName:        Array[1..48] of char;
     clientType:        NWOBJ_TYPE;
     clientName:        Array[1..48] of char;
     END;

TYPE TBinderyObject = RECORD
     ObjectName:        NWObjectName;
     ObjectID:          NWOBJ_ID;
     ObjectType:        NWOBJ_TYPE;
    END;

TYPE SERVER_AND_VCONSOLE_INFO = RECORD
     currentServerTime:         DWord;
     vconsoleVersion:           Byte;
     vconsoleRevision:          Byte;
     END;

TYPE NWFSE_SERVER_SET_CATEGORIES = RECORD
     serverTimeAndVConsoleInfo: SERVER_AND_VCONSOLE_INFO;
     reserved:                  WORD;
     numberOfSetCategories:     DWord;
     nextSequenceNumber:        DWord;
     categoryName:              Array[1..500] of Byte;
     END;

TYPE NWFSE_SERVER_SET_CMDS_INFO = RECORD
     serverTimeAndVConsoleInfo: SERVER_AND_VCONSOLE_INFO;
     reserved:                  WORD;
     numberOfSetCommands:       DWord;
     nextSequenceNumber:        DWord;
     setCmdType:                DWord;
     setCmdCategory:            DWord;
     setCmdFlags:               DWord;
     setNameAndValueInfo:       Array[1..500] of Char;
     END;

TYPE TNLM_INFO = RECORD
     identificationNum,         flags,
     NLMtype,                   parentID,
     majorVersion,              minorVersion,
     revision,                  year,
     month,                     day,
     allocAvailableBytes,       allocFreeCount,
     lastGarbageCollection,     messageLanguage,
     numOfReferencedPublics:    DWord
     end;

TYPE NWFSE_NLM_INFO = RECORD
     serverTimeAndVConsoleInfo: SERVER_AND_VCONSOLE_INFO;
     reserved:                  WORD;
     NLMInfo:                   TNLM_INFO;
     END;

TYPE TNWDSK_CACHE_STATS = Record
     systemElapsedTime        : DWORD;
     cacheBufferCount,
     cacheBufferSize,
     dirtyCacheBuffers        : WORD;
     cacheReadRequests,
     cacheWriteRequests,
     cacheHits,
     cacheMisses,
     physicalReadRequests,
     physicalWriteRequests    : DWORD;
     physicalReadErrors,
     physicalWriteErrors      : WORD;
     cacheGetRequests,
     cacheFullWriteRequests,
     cachePartialWriteRequests,
     backgroundDirtyWrites,
     backgroundAgedWrites,
     totalCacheWrites,
     cacheAllocations          : DWORD;
     thrashingCount,
     LRUBlockWasDirtyCount,
     readBeyondWriteCount,
     fragmentedWriteCount,
     cacheHitOnUnavailCount,
     cacheBlockScrappedCount   : WORD;
     end;

TYPE CACHE_COUNTERS = Record
     readExistingBlockCount,
     readExistingWriteWaitCount,
     readExistingPartialReadCount,
     readExistingReadErrorCount,
     writeBlockCount,
     writeEntireBlockCount,
     getDiskCount,
     getDiskNeedToAllocCount,
     getDiskSomeoneBeatMeCount,
     getDiskPartialReadCount,
     getDiskReadErrorCount,
     getAsyncDiskCount,
     getAsyncDiskNeedToAlloc,
     getAsyncDiskSomeoneBeatMe,
     errorDoingAsyncReadCount,
     getDiskNoReadCount,
     getDiskNoReadAllocCount,
     getDiskNoReadSomeoneBeatMeCount,
     diskWriteCount,
     diskWriteAllocCount,
     diskWriteSomeoneBeatMeCount,
     writeErrorCount,
     waitOnSemaphoreCount,
     allocBlockWaitForSomeoneCount,
     allocBlockCount,
     allocBlockWaitCount:          DWORD
     end;

TYPE CACHE_MEM_COUNTERS = Record
     originalNumOfCacheBuffers,
     currentNumOfCacheBuffers,
     cacheDirtyBlockThreshold,
     waitNodeCount,
     waitNodeAllocFailureCount,
     moveCacheNodeCount,
     moveCacheNodeFromAvailCount,
     accelerateCacheNodeWriteCount,
     removeCacheNodeCount,
     removeCacheNodeFromAvailCount:          DWORD;
     end;

TYPE CACHE_TREND_COUNTERS = Record
     numCacheChecks,
     numCacheHits,
     numDirtyCacheChecks,
     numDirtyCacheHits,
     cacheUsedWhileChecking,
     waitForDirtyBlocksDecreaseCount,
     allocBlockFromAvailCount,
     allocBlockFromLRUCount,
     allocBlockAlreadyWaiting,
     LRUSittingTime:          DWORD;
     end;

TYPE CACHE_INFO = Record
     maxByteCount,
     minNumOfCacheBuffers,
     minCacheReportThreshold,
     allocWaitingCount,
     numDirtyBlocks,
     cacheDirtyWaitTime,
     cacheMaxConcurrentWrites,
     maxDirtyTime,
     numOfDirCacheBuffers,
     cacheByteToBlockShiftFactor:          DWORD;
     end;


TYPE NWFSE_CACHE_INFO = Record
     serverTimeAndVConsoleInfo: SERVER_AND_VCONSOLE_INFO;
     reserved:                  WORD;
     cacheCounters:             CACHE_COUNTERS;
     cacheMemCounters:          CACHE_MEM_COUNTERS;
     cacheTrendCounters:        CACHE_TREND_COUNTERS;
     cacheInformation:          CACHE_INFO;
     end;

CONST FSE_NLM_NUMS_RETURNED_MAX = 125; { may be changed }

TYPE NWFSE_NLM_LOADED_LIST = RECORD
     serverTimeAndVConsoleInfo: SERVER_AND_VCONSOLE_INFO;
     reserved:                  WORD;
     numberNLMsLoaded:          DWord;
     NLMsInList:                DWord;
     NLMNums:                   Array[0..FSE_NLM_NUMS_RETURNED_MAX] of DWORD;
     end;


TYPE FSE_SERVER_INFO = RECORD
     replyCanceledCount,
     writeHeldOffCount,
     writeHeldOffWithDupRequest,
     invalidRequestTypeCount,
     beingAbortedCount,
     alreadyDoingReallocCount,
     deAllocInvalidSlotCount,
     deAllocBeingProcessedCount,
     deAllocForgedPacketCount,
     deAllocStillTransmittingCount,
     startStationErrorCount,
     invalidSlotCount,
     beingProcessedCount,
     forgedPacketCount,
     stillTransmittingCount,
     reExecuteRequestCount,
     invalidSequenceNumCount,
     duplicateIsBeingSentAlreadyCnt,
     sentPositiveAcknowledgeCount,
     sentDuplicateReplyCount,
     noMemForStationCtrlCount,
     noAvailableConnsCount,
     reallocSlotCount,
     reallocSlotCameTooSoonCount:          DWord;
     end;

TYPE FILE_SERVER_COUNTERS = RECORD
     tooManyHops,
     unknownNetwork,
     noSpaceForService,
     noReceiveBuffers,
     notMyNetwork:          Word;
     netBIOSProgatedCount,
     totalPacketsServiced,
     totalPacketsRouted:          DWord;
     end;

TYPE NWFSE_FILE_SERVER_INFO = RECORD
     serverTimeAndVConsoleInfo: SERVER_AND_VCONSOLE_INFO;
     reserved:                  WORD;
     NCPStationsInUseCount,
     NCPPeakStationsInUseCount,
     numOfNCPRequests,
     serverUtilization:         DWORD;
     ServerInfo:                FSE_SERVER_INFO;
     fileServerCounters:        FILE_SERVER_COUNTERS;
     end;

TYPE NWFSE_OS_VERSION_INFO = RECORD
     serverTimeAndVConsoleInfo: SERVER_AND_VCONSOLE_INFO;
     reserved:                  WORD;
     OSMajorVersion,            OSMinorVersion,
     OSRevisionNum,             accountingVersion,
     VAPVersion,                queueingVersion,
     securityRestrictionsLevel,
     bridgingSupport:           BYTE;
     maxNumOfVolumes,           numOfConnSlots,
     maxLoggedInConns,          maxNumOfNameSpaces,
     maxNumOfLans,              maxNumOfMediaTypes,
     maxNumOfProtocols,         maxMaxSubdirTreeDepth,
     maxNumOfDataStreams,       maxNumOfSpoolPrinters,
     serialNum:                 DWORD;
     applicationNum:            WORD;
     end;

TYPE NWFSE_ACTIVE_LAN_BOARD_LIST = RECORD
     serverTimeAndVConsoleInfo: SERVER_AND_VCONSOLE_INFO;
     reserved:                  WORD;
     maxNumOfLANs:              DWORD;
     LANLoadedCount:            DWORD;
     boardNums:                 Array[1..FSE_MAX_NUM_OF_LANS] of DWORD;
     end;

TYPE NWLAN_CONFIG = RECORD { NW 2.2 }
     Address:                   TAddress;
     boardInstalled:            BYTE;
     optionNumber:              BYTE;
     configurationText1:        Array[1..80] of Char;
     configurationText2:        Array[1..80] of Char;
     end;

TYPE LAN_CONFIG_INFO = RECORD
     DriverCFG_MajorVersion:    BYTE;
     DriverCFG_MinorVersion:    BYTE;
     DriverNodeAddress:         Array[1..6] of  BYTE;
     DriverModeFlags:           WORD;
     DriverBoardNum:            WORD;
     DriverBoardInstance:       WORD;
     DriverMaxSize:             DWORD;
     DriverMaxRecvSize:         DWORD;
     DriverRecvSize:            DWORD;
     Reserved1:                 Array[1..3] of  DWORD;

     DriverCardID:              WORD;
     DriverMediaID:             WORD;
     DriverTransportTime:       WORD;
     DriverReserved:            Array[1..16] of BYTE;
     DriverMajorVersion:        BYTE;
     DriverMinorVersion:        BYTE;
     DriverFlags:               WORD;
     DriverSendRetries:         WORD;
     DriverLink:                DWORD;
     DriverSharingFlags:        WORD;
     DriverSlot:                WORD;
     DriverIOPortsAndLengths:   Array[1..4] of  WORD;
     DriverMemDecode0:          DWORD;
     DriverLength0:             WORD;
     DriverMemDecode1:          DWORD;
     DriverLength1:             WORD;
     DriverInterrupt:           Array[1..2] of  BYTE;
     DriverDMAUsage:            Array[1..2] of  BYTE;
     Reserved2:                 Array[1..3] of  DWORD;
     DriverLogicalName:         Array[1..18] of Char;

     { this ?}
     DriverLinearMem:           Array[1..2] of  DWORD;
     DriverChannelNum:          WORD;
     DriverIOReserved:          Array[1..6] of  BYTE;
     WSTest:                    Array[1..200] of Char;
     { or
     DriverIOReserved:             Array[1..14] of Byte;
     DriverCardName:               Array[1..128] of Char;
     DriverMediaType:              Array[1..40] of Char;
     DriverCustomVars:             Array[1..180] of BYTE;
     }
     end;

TYPE NWFSE_LAN_CONFIG_INFO = RECORD
     serverTimeAndVConsoleInfo: SERVER_AND_VCONSOLE_INFO;
     reserved:                  WORD;
     LANConfigInfo:             LAN_CONFIG_INFO
     end;

TYPE NW_FRAGMENT = RECORD
   fragAddress:                    Pointer;
   fragSize1:                      DWORD;
   end;

TYPE LCONV = RECORD
   decimal_point      : Array[1..4] of char;
   thousands_sep      : Array[1..4] of char;
   grouping           : Array[1..4] of char;
   int_curr_symbol    : Array[1..8] of char;
   currency_symbol    : Array[1..4] of char;
   mon_decimal_point  : Array[1..4] of char;
   mon_thousands_sep  : Array[1..4] of char;
   mon_grouping       : Array[1..8] of char;
   positive_sign      : Array[1..4] of char;
   negative_sign      : Array[1..4] of char;
   int_frac_digits,
   frac_digits,
   p_cs_precedes,
   p_sep_by_space,
   n_cs_precedes,
   n_sep_by_space,
   p_sign_posn,
   n_sign_posn        : char;
   code_page          : integer;
   country_id         : integer;
   data_list_separator: Array[1..2] of char;
   date_separator     : Array[1..2] of char;
   time_separator     : Array[1..2] of char;
   time_format        : char;
   date_format        : integer;
   reserved           : Array[1..40] of char;
   end;

CONST
     NWSubAllocEnableBit      = $01;   {values for statusFlag in TNWVolExtendedInfo}
     NWCompressionEnabledBit  = $02;
     NWMigrationEnableBit     = $04;
     NWAuditingEnabledBit     = $08;
     NWReadOnlyEnableBit      = $10;

TYPE TNWVolExtendedInfo = Record
     volType                  : NWVOL_TYPE;
     statusFlag               : NWSTATUS;
     sectorSize,
     sectorsPerCluster,              volSizeInClusters,
     freeClusters,                   subAllocFreeableClusters,
     freeableLimboSectors,           nonfreeableLimboSectors,
     availSubAllocSectors,           nonuseableSubAllocSectors,
     subAllocClusters,               numDataStreams,
     numLimboDataStreams,            oldestDelFileAgeInTicks,
     numCompressedDataStreams,       numCompressedLimboDataStreams,
     numNoncompressibleDataStreams,  precompressedSectors,
     compressedSectors,              numMigratedDataStreams,
     migratedSectors,                clustersUsedByFAT,
     clustersUsedByDirs,             clustersUsedByExtDirs,
     totalDirEntries,                unusedDirEntries,
     totalExtDirExtants,             unusedExtDirExtants,
     extAttrsDefined,                extAttrExtantsUsed,
     directoryServicesObjectID,      volLastModifiedDateAndTime: NWNUM;
     END;

TYPE
   TRequestControl = RECORD
	   Len : WORD;
	   Fct : BYTE;
	   END;

   RequestBuffer   = ARRAY[0..576] of BYTE;
   ReplyBuffer	   = ARRAY[0..576] of BYTE;

   PathStr         = string[128];
   DirStr          = string[80];
   NameStr	   = String[15];
   ExtStr          = String[6];

   FilePath        = RECORD
                     Dir : DirStr;
                     Name: NameStr;
                     Ext : ExtStr
                     END;

VAR
   cCode: NWCCODE;

FUNCTION hex2(Number: BYTE): Str2;
FUNCTION hex4(Number: Word): Str4;
FUNCTION hex8(nr: LongInt):  Str8;
FUNCTION hex(VAR data; L: Byte): String;
FUNCTION Min(a, b: LongInt): LongInt;
Function Max(a, b: LongInt): LongInt;
FUNCTION UpperString(S: String): String;
FUNCTION ArrayToString(VAR A; Start, Len: Word): String;
FUNCTION ASCIIz(VAR A): String;
FUNCTION Str2PChar(VAR S: String): PChar;
PROCEDURE AddTermination(Var S: String);
FUNCTION  AreYouSure: Word;
PROCEDURE ShowError(ErrorCode: LongInt; ErrorMessage: String; Severity, Threshold: Byte);
PROCEDURE PressAnyKeyToContinue;
FUNCTION NW_Version(connID: NWCONN_HANDLE): Real;
FUNCTION Rights(R: WORD; NW_Ver: Real): Str8;
FUNCTION WriteFullName(connID: NWCONN_HANDLE; oName: PChar; oType: NWOBJ_TYPE; VAR FullName): NWCCODE;
FUNCTION GetMailboxFromID(connID: NWCONN_HANDLE; OID: LongInt; objectType: NWOBJ_TYPE): Str32;
FUNCTION CreateMailboxFromID(connID: NWCONN_HANDLE; OID: LongInt; objectType: NWOBJ_TYPE; Threshold: BYTE): NWCCODE;
PROCEDURE CopyTrustees(sourceConnID, targetConnID: NWCONN_HANDLE; srcID, dstID: NWOBJ_ID; srcNWVers, dstNWVers: Real);
FUNCTION AddAndSyncMembership(connID: NWCONN_HANDLE; OName: PChar;   OType: NWOBJ_TYPE; PropName: PChar;
                                                     MmbName: PChar; MmbType: NWOBJ_TYPE): NWCCODE;
FUNCTION DelAndSyncMembership(connID: NWCONN_HANDLE; OName: PChar;   OType: NWOBJ_TYPE; PropName: PChar;
                                                     MmbName: PChar; MmbType: NWOBJ_TYPE): NWCCODE;
PROCEDURE ListBoxAddObjects(Sender: TObject; connID: NWCONN_HANDLE; VAR TargetList: TListBox;
                            OPattern: PChar; OType: NWOBJ_TYPE);
FUNCTION  NDSattach(VAR conn: NWCONN_HANDLE; srv: String): NWCCODE;
FUNCTION  NDSDetach(FS: String): NWCCODE;

PROCEDURE FSplit(Path: String; VAR D: DirStr; VAR N: NameStr; VAR E: ExtStr);
FUNCTION CopyFile(Source : String; Destination : String): NWCCODE;
FUNCTION NCPCall(connID: NWCONN_HANDLE; NCPFct: Byte; ReqLen: WORD; VAR Req; RepLen: WORD; VAR Rep): NWCCODE;
FUNCTION ReturnBinderyContext(connID: NWCONN_HANDLE; VAR BContext: String): NWCCODE;
FUNCTION ReturnTreeName(connID: NWCONN_HANDLE; VAR TreeName : String): NWCCODE;
FUNCTION ReloadDSSoftware(connID: NWCONN_HANDLE; VAR Status: int4): NWCCODE;
Function NW40_LoadAnNLM(connID: NWCONN_HANDLE; VAR RPCCode: LongInt; NLMLoadOptions: LongInt; PathAndName: String): NWCCODE;
Function NW40_UnloadAnNLM(connID: NWCONN_HANDLE; VAR RPCCode: LongInt; PathAndName: String): NWCCODE;
Function NW40_MountVolume(connID: NWCONN_HANDLE; VAR RPCCode: LongInt; VolumeName: String): NWCCODE;
Function NW40_DismountVolume(connID: NWCONN_HANDLE; VAR RPCCode: LongInt; VolumeName: String): NWCCODE;
Function NW40_AddNameSpaceToVolume(connID:NWCONN_HANDLE; VAR RPCCode:LongInt; VolumeName:String; NameSpaceName:String):NWCCODE;
Function NW40_ExecuteNCFFile(connID: NWCONN_HANDLE; VAR RPCCode: LongInt; PathAndName: String): NWCCODE;
Function NW40_SetSetCommandValue(connID: NWCONN_HANDLE; VAR RPCCode: LongInt; SetCommandName: String;
                                 LValue: LongInt; SValue: String): NWCCODE;


FUNCTION NWAddObjectToSet(connID: NWCONN_HANDLE;
  objectName:      PChar;
  objectType:      NWOBJ_TYPE;
  propertyName:    PChar;
  memberName:      PChar;
  memberType:      NWOBJ_TYPE): NWCCODE;

FUNCTION NWAddTrustee(connID: NWCONN_HANDLE;
  dirHandle:       NWDIR_HANDLE;
  path:            PChar;
  trusteeID:       NWOBJ_ID;
  rightsMask:      NWRIGHTS_MASK): NWCCODE;

FUNCTION NWAddTrusteeToDirectory(connID: NWCONN_HANDLE;
  dirHandle:       NWDIR_HANDLE;
  path:            PChar;
  trusteeID:       NWOBJ_ID;
  rightsMask:      NWACCESS_RIGHTS): NWCCODE;

FUNCTION NWAttachToFileServer(
   serverName:     PChar;
   scopeFlag:      NWLOCAL_SCOPE;                          { (IN)  for Novell use only; must be 0 }
   VAR newConnID:  NWCONN_HANDLE): NWCCODE;                { (OUT) new conn handle, if attachment was successful }

FUNCTION NWBroadcastToConsole(connID: NWCONN_HANDLE;
   msg:            PChar): NWCCODE;

FUNCTION NWCallsInit(PtrIn, PtrOut: Pointer): NWCCODE;

FUNCTION NWChangeObjectPassword(connID: NWCONN_HANDLE;
  ObjectName:      PChar;
  ObjectType:      Word;
  oldPassword:     PChar;
  newPassword:     PChar): NWCCODE;

FUNCTION NWChangeObjectSecurity(connID: NWCONN_HANDLE;
  ObjectName:      PChar;
  ObjectType:      Word;
  newSecurity:     NWFLAGS): NWCCODE;

FUNCTION NWCheckConsolePrivileges(connID: NWCONN_HANDLE): NWCCODE;

FUNCTION NWCheckNetWareVersion(connID: NWCONN_HANDLE;
   VAR minVersion, minSubVersion, minRevision, minSFT, minTTS:     WORD;    { (IN)  Indicate minimum version required }
   VAR compatibilityFlag: BYTE): NWCCODE;                    { (OUT) Points to a flag indicating compatibility:
                                                                     0x00   COMPATIBLE         0x01   VERSION_TOO_LOW
                                                                     0x02   SFT_LEVEL_TOO_LOW  0x04   TTS_LEVEL_TOO_LOW }
Function NWClearPhysicalRecord(
   fileHandle:             Word;
   recordStartOffset:      LongInt;
   recordLength: LongInt): NWCCODE;

Function NWCreateDirectory(connID: NWCONN_HANDLE;
  dirHandle:       NWDIR_HANDLE;
  path:            PChar;
  accessMask:      NWACCESS_RIGHTS): NWCCODE;

FUNCTION NWCreateObject(connID: NWCONN_HANDLE;
  objectName:      PChar;
  objectType:      Word;
  objectFlags:     Byte;
  objectSecurity:  Byte): NWCCODE;

FUNCTION NWCreateProperty(connID: NWCONN_HANDLE;
  objectName:      PChar;
  objectType:      Word;
  propertyName:    PChar;
  propertyFlags:   NWFLAGS;
  propertySecurity:NWFLAGS): NWCCODE;

FUNCTION NWCreateQueue(connID: NWCONN_HANDLE;
  objectName:      PChar;
  objectType:      Word;
  dirPath:         NWDIR_ID;
  pathName:        PChar;
  VAR queueID:     NWOBJ_ID): NWCCODE;

Function NWDeleteObject(connID: NWCONN_HANDLE;
  ObjectName:      PChar;
  ObjectType:      Word): NWCCODE;

FUNCTION NWDeleteObjectFromSet(connID: NWCONN_HANDLE;
  objectName:      PChar;
  objectType:      NWOBJ_TYPE;
  propertyName:    PChar;
  memberName:      PChar;
  memberType:      NWOBJ_TYPE): NWCCODE;

FUNCTION NWDeleteProperty(connID: NWCONN_HANDLE;
  objectName:      PChar;
  objectType:      Word;
  propertyName:    PChar): NWCCODE;

FUNCTION NWDestroyQueue(connID: NWCONN_HANDLE;
  queueID:         NWOBJ_ID): NWCCODE;

FUNCTION NWDetachFromFileServer(connID: NWCONN_HANDLE): NWCCODE;
         { disconnect someone else; needs operator rights }

FUNCTION NWDisableFileServerLogin(connID: NWCONN_HANDLE): NWCCODE;

FUNCTION NWDisableTTS(connID: NWCONN_HANDLE): NWCCODE;

FUNCTION NWDownFileServer(connID: NWCONN_HANDLE;
   forceFlag:      NWFLAGS): NWCCODE;

FUNCTION NWDSAuthenticate(connID: NWCONN_HANDLE;
  optionsFlag:     NWDS_FLAGS;                     { (IN)  reserved; pass in a zero (0)}
  VAR sessionKey:  NWDS_SESSION_KEY): NWDSCCODE;   { (IN)  reserved; pass in NULL }

FUNCTION  NWDSGetConnectionSlot(
   connType:       NWCONN_TYPE;     {SYSTEM_LOCK = hard allocation;
                                     TASK_LOCK = soft allocation;
                                     TASK_DISCONNECT = no resource allocation}
   transType:      NWTRAN_TYPE;     {Indicates transport protocol for this connection; for example, IPX=1}
   transLen:       NWLEN;           {Indicates length of the transport buffer; for example, IPX=12}
   VAR transBuf:   NWSERVER_ADDR;   {Points to transport dependent address information (12 bytes for IPX)}
   VAR conn:       NWCONN_HANDLE): NWCCODE;

FUNCTION NWDSLockConnection(connID: NWCONN_HANDLE): NWCCODE;

FUNCTION NWDSUnlockConnection(connID: NWCONN_HANDLE): NWCCODE;

FUNCTION NWEnableFileServerLogin(connID: NWCONN_HANDLE): NWCCODE;

FUNCTION NWEnableTTS(connID: NWCONN_HANDLE): NWCCODE;

FUNCTION NWFreeConnectionSlot(connID: NWCONN_HANDLE;
   disconnectType: NWFLAGS): NWCCODE;  { disconnectType can be SYSTEM_DISCONNECT or TASK_DISCONNECT }

FUNCTION NWFreeUnicodeTables: int;

FUNCTION NWGetActiveLANBoardList(connID: NWCONN_HANDLE;
  startNum                  : DWORD;  { start with 0 }
  VAR fseActiveLANBoardList : NWFSE_ACTIVE_LAN_BOARD_LIST): NWCCODE;  { bug in function? }

FUNCTION NWGetBinderyAccessLevel(connID: NWCONN_HANDLE;
  VAR accessLevel: NWFLAGS;
  VAR objectID:    NWOBJ_ID): NWCCODE;

FUNCTION NWGetCacheInfo(connID: NWCONN_HANDLE;
  VAR fseCacheInfo: NWFSE_CACHE_INFO): NWCCODE;

FUNCTION NWGetConnectionHandle(
  ServerName:      PChar;                                  { (IN)  name of the target NetWare server }
  mode:            Word;                                   { (IN)  for Novell use only; must be 0 }
  VAR ConnID:      Word;                                   { (OUT) Points to conn handle }
  connScope:       Pointer): NWCCODE;                      { (IN)  for Novell use only; must be NULL }

FUNCTION NWGetConnectionIDFromName(
  nameLen:         DWORD;                                  { (IN)  length of the server name }
  name:            PChar;                                  { (IN)  server name to look up in the connection table }
  VAR conn:        NWCONN_HANDLE): NWCCODE;                { (OUT) connection handle to the server name }

FUNCTION NWGetConnectionInformation(connID: NWCONN_HANDLE;
  connNumber:      NWCONN_NUM;
  objectName:      PChar;
  VAR objectType:  NWOBJ_TYPE;
  VAR objectID:    NWOBJ_ID;
  VAR loginTime:   NWLOGIN_TIME): NWCCODE;

FUNCTION NWGetConnectionList(
   mode:                NWLOCAL_MODE;     { 0 }
   VAR connListBuffer:  TNWConnHandleList;
   connListSize:        NWSTRUCT_SIZE;
   VAR numConnections:  NWNUMBER): NWCCODE;

FUNCTION NWGetConnectionNumber(connID: NWCONN_HANDLE;
  VAR connNumber:       NWCONN_NUM): NWCCODE;

FUNCTION NWGetConnectionStatus(connID: NWCONN_HANDLE;
  VAR connInfo:    CONNECT_INFO;
  connInfoSize:    NWSTRUCT_SIZE): NWCCODE;

FUNCTION NWGetDefaultConnectionID(VAR ConnID: NWCONN_HANDLE): NWCCODE;

FUNCTION NWGetDiskCacheStats(connID: NWCONN_HANDLE;            { NW 2.x }
   VAR statBuffer:  TNWDSK_CACHE_STATS): NWCCODE;

FUNCTION NWGetExtendedVolumeInfo(connID: NWCONN_HANDLE;
   volNumber:      WORD;
   VAR volInfo:    TNWVolExtendedInfo): NWCCODE;

FUNCTION  NWGetFileServerInfo(connID: NWCONN_HANDLE;
   VAR fseFileServerInfo:  NWFSE_FILE_SERVER_INFO): NWCCODE;


FUNCTION NWGetFileServerInformation(connID: NWCONN_HANDLE;
  serverName:                PChar;
  VAR majorVersion,
      minorVersion,
      revision:              NWSERVICE_VERSION;
  VAR maxConnections,
      maxConnectionsUsed,
      connectionsInUse,
      numVolumes:            NWNUMBER;
  VAR SFTLevel,
      TTSLevel:              NWSUPPORT_LEVEL): NWCCODE;

FUNCTION  NWGetFileServerLoginStatus(connID: NWCONN_HANDLE;
   VAR loginEnabledFlag:     NWFLAGS): NWCCODE;

FUNCTION  NWGetFSLANDriverConfigInfo(connID: NWCONN_HANDLE;   { NW 2.2 }
   lanBoardNumber:           NWLAN_NUM;
   VAR lanConfig:            NWLAN_CONFIG): NWCCODE;

FUNCTION NWGetLANConfigInfo(connID: NWCONN_HANDLE;
   lanBoardNumber:           DWORD;
   VAR fseLANConfigInfo:     NWFSE_LAN_CONFIG_INFO): NWCCODE;

FUNCTION NWGetNearestDirectoryService(
   VAR connID: NWCONN_HANDLE): NWCCODE;

FUNCTION NWGetNetworkSerialNumber(connID: NWCONN_HANDLE;
   VAR serialNum:            DWORD;
   VAR appNum:               WORD): NWCCODE;

FUNCTION NWGetNLMInfo(connID: NWCONN_HANDLE;
   NLMNum:                   DWORD;
   fileName, Name, copyright:PChar;
   VAR fseNLMInfo:           NWFSE_NLM_INFO): NWCCODE;

FUNCTION NWGetNLMLoadedList(connID: NWCONN_HANDLE;
   startNum:                 DWORD;
   VAR fseNLMLoadedList:     NWFSE_NLM_LOADED_LIST): NWCCODE;

FUNCTION NWGetNumConnections(
  VAR numConnections:        NWNUMBER): NWCCODE;

FUNCTION NWGetObjectConnectionNumbers(connID: NWCONN_HANDLE;
  objectName:                PChar;
  objectType:                Word;
  VAR numConnections:        WORD;
  VAR connList:              TConnList;
  maxConn:                   Word): NWCCODE;

Function NWGetObjectID(connID: NWCONN_HANDLE;
  objectName:          PChar;
  objectType:          Word;
  VAR objectID:        NWOBJ_ID): NWCCODE;

Function NWGetObjectName(connID: NWCONN_HANDLE;
  objectID:            NWOBJ_ID;
  VAR objectName:      NWObjectName;
  VAR objectType:      NWOBJ_TYPE): NWCCODE;

Function NWGetOSVersionInfo(connID: NWCONN_HANDLE;
  VAR fseOSVersionInfo: NWFSE_OS_VERSION_INFO): NWCCODE;

Function NWGetPrimaryConnectionID(VAR conn: NWCONN_HANDLE): NWCCODE;

Function NWGetServerSetCategories(connID: NWCONN_HANDLE;
   startNum:                     DWord;
   VAR fseServerSetCategories:   NWFSE_SERVER_SET_CATEGORIES): NWCCODE;

Function NWGetServerSetCommandsInfo(connID: NWCONN_HANDLE;
   startNum:                     DWord;
   VAR fseServerSetCmdsInfo:     NWFSE_SERVER_SET_CMDS_INFO): NWCCODE;

{ Function NWGetTTSStats(connID: NWCONN_HANDLE;
   VAR ttsStats:                 TTS_STATS): NWCCODE;        NW 2.2 }

Function NWGetVolumeInfoWithNumber(connID: NWCONN_HANDLE;
  volumeNumber:                  Byte;
  volumeName:                    PChar;
  VAR totalBlocks:               Word;
  VAR sectorsPerBlock:           Word;
  VAR availableBlocks:           Word;
  VAR totalDirectoryEntries:     Word;
  VAR availableDirectoryEntries: Word;
  VAR volumeIsRemovable: Word):  NWCCODE;

FUNCTION NWInitUnicodeTables(countryCode: int;           {  (IN) country or nation identification number }
   codePage: int): int;                                  {  (IN) Character Encoding Scheme (CES) ID number }

FUNCTION NWIsManager(connID: NWCONN_HANDLE): NWCCODE;

FUNCTION NWLlocaleconv(VAR lconvPtr: LCONV): NWAPI;       { (OUT) Points to LCONV }

FUNCTION NWLoginToFileServer(connID: NWCONN_HANDLE;
  ObjectName:      PChar;
  ObjectType:      Word;
  password:        PChar): NWCCODE;

Function NWLogPhysicalRecord(
   fileHandle:     Word;
   recordStartOffset: LongInt;
   recordLength:   LongInt;
   lockDirective:  Byte;
   timeoutLimit:   Word): NWCCODE;

Function  NWLsetlocale(category: int;            {  (IN) Indicates the locale categories. }
  locale: PChar): NWAPI;                         {  (IN) Points to a locale string        }

Function NWReadPropertyValue(connID: NWCONN_HANDLE;
  objectName:            PChar;
  objectType:            NWOBJ_TYPE;
  propertyName:          PChar;
  dataSetIndex:          NWSEGMENT_NUM;
  VAR dataBuffer:        TPropSegm;
  VAR moreFlag:          NWFLAGS;
  VAR propertyFlags:     NWFLAGS): NWCCODE;

Function NWRenameObject(connID: NWCONN_HANDLE;
  oldObjectName:         PChar;
  newObjectName:         PChar;
  objectType:            NWOBJ_TYPE): NWCCODE;

FUNCTION NWRequest(connID: NWCONN_HANDLE;
   NWfunction:          WORD;
   numReqFrags:         WORD;
   VAR reqFrags:        NW_FRAGMENT;
   numReplyFrags:       WORD;
   VAR replyFrags:      NW_FRAGMENT): NWCCODE;

Function  NWScanObject(connID: NWCONN_HANDLE;
  searchName:            PChar;
  searchType:            NWOBJ_TYPE;
  VAR objectID:          NWOBJ_ID;
  VAR ObjectName:        NWObjectName;
  VAR objectType:        NWOBJ_TYPE;
  VAR hasPropertiesFlag: NWFLAGS;
  VAR objectFlags:       NWFLAGS;
  VAR objectSecurity:    NWFLAGS): NWCCODE;

Function  NWScanObjectTrusteePaths(connID: NWCONN_HANDLE;
  objectID:                        NWOBJ_ID;
  volumeNumber:                    NWVOL_NUM;
  VAR sequenceNumber:              NWTRUSTEE_SEQUENCE_NUM;
  VAR trusteeAccessRights:         NWFLAGS;
  path:                            PChar): NWCCODE;

Function NWScanProperty(connID: NWCONN_HANDLE;
  objectName:            PChar;
  objectType:            NWOBJ_TYPE;
  searchPropertyName:    PChar;
  VAR sequenceNumber:    NWSEQUENCE;
  VAR propertyName:      NWPropertyName;
  VAR propertyFlags:     NWFLAGS;
  VAR propertySecurity:  NWFLAGS;
  VAR hasValueFlag:      NWFLAGS;
  VAR moreFlag:          NWFLAGS): NWCCODE;

{Function NWSetLockMode(lockMode: Byte): NWCCODE; not required}

FUNCTION NWSetPreferredServer(ConnID: NWCONN_HANDLE): NWCCODE;

FUNCTION NWTTSIsAvailable(ConnID: NWCONN_HANDLE): NWCCODE;

FUNCTION NWVerifyObjectPassword(connID: NWCONN_HANDLE;
  ObjectName:            PChar;
  ObjectType:            Word;
  password:              PChar): NWCCODE;

Function NWWritePropertyValue(connID: NWCONN_HANDLE;
  objectName:            PChar;
  objectType:            NWOBJ_TYPE;
  searchPropertyName:    PChar;
  segmentNumber:         NWSEGMENT_NUM;
  VAR segmentData:       TPropSegm;
  moreSegments:          NWFLAGS): NWCCODE;


implementation

FUNCTION hex2(Number: BYTE): Str2;
CONST  hexa : Array[0..15] of Char = '0123456789ABCDEF';
BEGIN
     hex2[0] := #2;
     hex2[1] := hexa[Number SHR 4];
     hex2[2] := hexa[Number AND $0F];
END;

FUNCTION  hex4(Number: Word): Str4;
BEGIN
     hex4 :=  hex2(hi(Number)) + hex2(lo(Number));
END;

FUNCTION  hex8(nr: LongInt): Str8;
VAR DW : Record
         WHi, WLo : Word
         END absolute nr;
BEGIN
   hex8 := hex4(Swap(DW.WHi)) + hex4(Swap(DW.WLo))
END;

FUNCTION  hex(VAR data; L: Byte): String;
VAR A : Array[1..128] of BYTE absolute data;
    S : String;
    i : Byte;
BEGIN
   S := '';
   For i := 1 to L Do S := S + hex2(A[i]);
   hex := S;
END;

Function Min(a, b: LongInt): LongInt;
begin
   if a<b then Min := a  else Min := b;
end;

Function Max(a, b: LongInt): LongInt;
begin
   if a>b then Max := a  else Max := b;
end;

FUNCTION  UpperString(S: String): String;
VAR i  : Byte;
BEGIN
   For i := 1 to Length(S) DO S[i] := UpCase(S[i]);
   UpperString := S
END;

FUNCTION  ArrayToString(VAR A; Start, Len: Word): String;
VAR i: Byte;
    S: String;
    x: Array[0..1000] of Char absolute A;
BEGIN
   if Len>255 then Len:=255;
   FillChar(S, SizeOf(S), 0);
   Move(x[Start], S[1], Len);
   S[0] := #255;
   for i := 255 downto 1 do
       IF S[i]=#0 THEN S[0]:=Char(Pred(i));
   ArrayToString := S
END;

FUNCTION  ASCIIz(VAR A): String;
VAR i: Byte;
    S: String;
BEGIN
   Move(A, S[1], 255);
   S[0] := #255;
   for i := 255 downto 1 do
       IF S[i]=#0 THEN S[0]:=Char(Pred(i));
   ASCIIz := S
END;

PROCEDURE AddTermination(Var S: String);
begin
   S[Succ(Length(S))] := #0;
end;

FUNCTION Str2PChar(VAR S: String): PChar;
begin
   AddTermination(S);
   Str2PChar := @S[1];
end;

FUNCTION AreYouSure: Word;
BEGIN  { if OK then return value is 'mrOK' }
   AreYouSure := MessageDlg('Are you sure ?', mtConfirmation, mbOKCancel, 0);
END;

PROCEDURE ShowError(ErrorCode: LongInt; ErrorMessage: String; Severity, Threshold: Byte); {Success, Warning, Err, Fatal}
VAR SeverityStr : Str8;
    OldCursor   : TCursor;
    ErrorCodeW  : WORD;
BEGIN
   OldCursor     := Screen.Cursor;
   Screen.Cursor := crDefault;
   Move(ErrorCode, ErrorCodeW, 2);
   if (ErrorCodeW > 0) and (Severity>=Threshold) THEN BEGIN
      {$IFDEF Windows}
      Case Severity of
           Warning: SeverityStr := 'Warning';
           Err:     SeverityStr := 'Error';
           Fatal:   SeverityStr := 'Fatal';
           end;
      MyShowMessage(SeverityStr+' 0x'+hex4(ErrorCodeW)+': '+ErrorMessage);
      {$else}
      WriteLn(SeverityStr+' 0x', hex4(ErrorCodeW), ': ', ErrorMessage);
      {$endIF}
      IF Severity=Fatal THEN Halt(1)
      END;
   Screen.Cursor := OldCursor;
END;

PROCEDURE PressAnyKeyToContinue;
VAR C: Char;
BEGIN
   Write('Press any key to continue');
   C := ReadKey;
   WriteLn;
   IF C = #27 THEN Halt(1)
END;

Function NW_Version(connID: NWCONN_HANDLE): Real;
VAR serverName:            PChar;
    majorVersion:          NWSERVICE_VERSION;
    minorVersion:          NWSERVICE_VERSION;
    revision:              NWSERVICE_VERSION;
    maxConnections,
    maxConnectionsUsed,
    connectionsInUse,
    numVolumes:            NWNUMBER;
    SFTLevel,
    TTSLevel:              NWSUPPORT_LEVEL;
{   cCode:                 WORD; }
BEGIN
   serverName := StrAlloc(255);
   cCode := NWGetFileServerInformation(connID, serverName, majorVersion,
               minorVersion, revision, maxConnections, maxConnectionsUsed,
               connectionsInUse, numVolumes, SFTLevel, TTSLevel);
   if cCode = 0 THEN NW_Version := majorVersion + (0.01*minorVersion)
                else NW_Version := 0;
   StrDispose(serverName);
END;

FUNCTION  ShowBits(zahl: BYTE; Muster0, Muster1: Str8): Str8;
VAR i : BYTE;
BEGIN
FOR i:=1 to 8  DO BEGIN
    IF  (Zahl MOD 2)=0 THEN Muster1[i]:=Muster0[i];
    Zahl := Zahl DIV 2
    END;
ShowBits := Muster1
END;

FUNCTION  SelectBits(zahl: BYTE; Muster: Str8): Str8;
BEGIN
	SelectBits := ShowBits(zahl, '        ', Muster)
END;

FUNCTION Rights(R: WORD; NW_Ver: Real): Str8;
BEGIN
IF NW_Ver < 2.2 THEN Rights:=SelectBits(R,'RWOCDPSM')
   ELSE BEGIN
   Rights := '        ';
   IF (R and TR_Supervisor)>0 THEN Rights[1]:='S';
   IF (R and TR_READ)>0       THEN Rights[2]:='R';
   IF (R and TR_WRITE)>0      THEN Rights[3]:='W';
   IF (R and TR_CREATE)>0     THEN Rights[4]:='C';
   IF (R and TR_ERASE)>0      THEN Rights[5]:='E';
   IF (R and TR_MODIFY)>0     THEN Rights[6]:='M';
   IF (R and TR_FILE)>0       THEN Rights[7]:='F';
   IF (R and TR_ACCESS)>0     THEN Rights[8]:='A'
   END
END;

FUNCTION GetMailboxFromID(connID: NWCONN_HANDLE; OID: LongInt; objectType: NWOBJ_TYPE): Str32;
VAR SX: Str8;  { sample: GetMailboxFromID(CID, 'SYS:MAIL', ID); }
BEGIN
	SX := hex8(OID);
	WHILE SX[1]='0' DO Delete(SX,1,1);
        Case Swap(objectType) of
             OT_User:         GetMailboxFromID := 'SYS:MAIL\'  +SX;
             OT_Print_Server: GetMailboxFromID := 'SYS:SYSTEM\'+SX;
             OT_Print_Queue:  GetMailboxFromID := 'SYS:SYSTEM\'+SX+'.QDR';
             else             GetMailboxFromID := 'SYS:MAIL\'  +SX;
             end; {case}
END;

FUNCTION CreateMailboxFromID(connID: NWCONN_HANDLE; OID: LongInt; objectType: NWOBJ_TYPE; Threshold: BYTE): NWCCODE;
VAR MailPath: Str24;
{    cCode:    NWCCODE; }
BEGIN
    MailPath := GetMailboxFromID(connID, OID, objectType);
    AddTermination(MailPath);
    cCode := NWCreateDirectory(connID, 0, @MailPath[1], TA_All);
    ShowError(cCode, 'NWCreateDir '+StrPas(@MailPath[1]), Warning, Threshold);
    cCode := NWAddTrusteeToDirectory(connID, 0, @MailPath[1], OID, (TA_All-TA_Access));
    ShowError(cCode, 'NWAddTrustee to'+StrPas(@MailPath[1]), Warning, Threshold);
    CreateMailboxFromID := cCode;
END;

FUNCTION WriteFullName(connID: NWCONN_HANDLE; oName: PChar; oType: NWOBJ_TYPE; VAR FullName): NWCCODE;
VAR { cCode:    NWCCODE; }
    Data:     TPropSegm absolute FullName;
BEGIN
   cCode := NWCreateProperty(connID, oName, oType, 'IDENTIFICATION', BF_STATIC+BF_ITEM, BS_LOGGED_READ+BS_SUPER_WRITE);
   cCode := NWWritePropertyValue(connID, oName, oType, 'IDENTIFICATION', 1, Data, 0);
   WriteFullName := cCode;
END;

Procedure Fix3xRights(NWVers: Real; VAR R: Word);
BEGIN  { when using rights from NW v2.x, add open rights }
   IF (NWVers>=3) and (((R and TR_Write)>0) or ((R and TR_Read)>0)) THEN R := (R OR TA_Open);
END;

PROCEDURE CopyTrustees(sourceConnID, targetConnID: NWCONN_HANDLE; srcID, dstID: NWOBJ_ID; srcNWVers, dstNWVers: Real);
VAR
  volumeNumber:                NWVOL_NUM;
  sequenceNumber:              NWTRUSTEE_SEQUENCE_NUM;
  trusteeAccessRights_B:       NWACCESS_RIGHTS;
  trusteeAccessRights_W:       NWDIR_TRUSTEE_RIGHTS;
  path:                        PChar;
{ cCode:                       NWCCODE; }
  temp:                        Boolean;
BEGIN
    path := StrAlloc(255);
    For volumeNumber   := 0 to 31 DO Begin
        sequenceNumber := NWTRUSTEE_SEQUENCE_NUM(-1);
        repeat
           cCode := NWScanObjectTrusteePaths(sourceConnID, srcID,
                    volumeNumber, sequenceNumber, trusteeAccessRights_B, path);
           if ( (path[1]=#0) or (sequenceNumber=0)) THEN cCode := $FFFF;
           if cCode=0 THEN BEGIN
              trusteeAccessRights_W := trusteeAccessRights_B;
              Fix3xRights(dstNWVers, trusteeAccessRights_W);
              trusteeAccessRights_B := trusteeAccessRights_W;

              temp := (StrLen(path)>9);
              temp := (StrLen(path)<18);
              temp := (StrLComp(path, 'SYS:MAIL/', 9)=0);
              temp := (trusteeAccessRights_B = $DF);

              if not ((StrLen(path)>9)                   and (StrLen(path)<18) and
                      (StrLComp(path, 'SYS:MAIL/', 9)=0) and (trusteeAccessRights_B = $DF)) then
                 cCode := NWAddTrusteeToDirectory(targetConnID, 0, path, dstID, trusteeAccessRights_B);
              END;
           until (cCode<>0);
        END;
    StrDispose(path);
END;


FUNCTION WS_NWAddObjectToSet(connID: NWCONN_HANDLE; objectName: PChar; objectType: NWOBJ_TYPE; propertyName: PChar;
                                     memberName: PChar; memberType: NWOBJ_TYPE): NWCCODE;
begin
    cCode := NWAddObjectToSet(connID, objectName, objectType, propertyName, memberName, memberType);
    if (cCode = $89FB) then begin { no such property }
       cCode := NWCreateProperty(connID, objectName, objectType, propertyName,
                BF_SET OR BF_STATIC, BS_LOGGED_READ OR BS_SUPER_WRITE);
       cCode := NWAddObjectToSet(connID, objectName, objectType, propertyName, memberName, memberType);
       end;
    WS_NWAddObjectToSet := cCode;
end;


FUNCTION  AddAndSyncMembership(connID: NWCONN_HANDLE; OName: PChar;   OType: NWOBJ_TYPE; PropName: PChar;
                                                      MmbName: PChar; MmbType: NWOBJ_TYPE): NWCCODE;
begin
   if StrComp(PropName, 'GROUPS_I''M_IN')=0  then begin
      cCode := WS_NWAddObjectToSet(connID, MmbName, MmbType, 'GROUP_MEMBERS',   OName, OType);
      cCode := WS_NWAddObjectToSet(connID, OName, OType,     'SECURITY_EQUALS', MmbName, MmbType);
      end
       else
   if StrComp(PropName, 'GROUP_MEMBERS')=0  then begin
      cCode := WS_NWAddObjectToSet(connID, MmbName, MmbType, 'GROUPS_I''M_IN',  OName, OType);
      cCode := WS_NWAddObjectToSet(connID, MmbName, MmbType, 'SECURITY_EQUALS', OName, OType);
      end
       else
   if (StrComp(PropName, 'SECURITY_EQUALS')=0) and (MmbType = Swap(OT_USER_GROUP))  then begin
      cCode := WS_NWAddObjectToSet(connID, MmbName, MmbType, 'GROUP_MEMBERS',   OName, OType);
      cCode := WS_NWAddObjectToSet(connID, OName, OType,     'GROUPS_I''M_IN',  MmbName, MmbType);
      end;
   AddAndSyncMembership := WS_NWAddObjectToSet(connID, OName, OType, PropName,  MmbName, MmbType);
end;

FUNCTION  DelAndSyncMembership(connID: NWCONN_HANDLE; OName: PChar;   OType: NWOBJ_TYPE; PropName: PChar;
                                                      MmbName: PChar; MmbType: NWOBJ_TYPE): NWCCODE;
begin
   if StrComp(PropName, 'GROUPS_I''M_IN')=0  then begin
      cCode := NWDeleteObjectFromSet(connID, MmbName, MmbType, 'GROUP_MEMBERS',   OName, OType);
      cCode := NWDeleteObjectFromSet(connID, OName, OType,     'SECURITY_EQUALS', MmbName, MmbType);
      end
       else
   if StrComp(PropName, 'GROUP_MEMBERS')=0  then begin
      cCode := NWDeleteObjectFromSet(connID, MmbName, MmbType, 'GROUPS_I''M_IN',  OName, OType);
      cCode := NWDeleteObjectFromSet(connID, MmbName, MmbType, 'SECURITY_EQUALS', OName, OType);
      end
       else
   if (StrComp(PropName, 'SECURITY_EQUALS')=0)  and (MmbType = Swap(OT_USER_GROUP))  then begin
      cCode := NWDeleteObjectFromSet(connID, MmbName, MmbType, 'GROUP_MEMBERS',   OName, OType);
      cCode := NWDeleteObjectFromSet(connID, OName, OType,     'GROUPS_I''M_IN',  MmbName, MmbType);
      end;
   DelAndSyncMembership := NWDeleteObjectFromSet(connID, OName, OType, PropName,      MmbName, MmbType);
end;


procedure ListBoxAddObjects(Sender: TObject; connID: NWCONN_HANDLE;
                       VAR TargetList: TListBox; OPattern: PChar; OType: NWOBJ_TYPE);
VAR   NewPos:            Integer;
      UserName:          String;
      Obj:               scanObjectRep;
begin
     Screen.Cursor := crHourGlass;
     Obj.objectID := -1;
     Repeat
     cCode := NWScanObject(connID, OPattern, OType, Obj.objectID, Obj.ObjectName,
              Obj.ObjectType, Obj.ObjecthasProperties, Obj.ObjectFlag, Obj.ObjectSecurity);
     IF cCode = 0 THEN BEGIN
        UserName := StrPas(@Obj.ObjectName);
        NewPos := TargetList.Items.IndexOf(UserName);
        IF (NewPos=-1) THEN NewPos := TargetList.Items.Add(UserName);
        IF (NewPos=-1) THEN TargetList.ItemIndex := NewPos;
        END;
     Until cCode > 0;
     Screen.Cursor := crDefault;
end;

FUNCTION  NDSattach(VAR conn: NWCONN_HANDLE; srv: String): NWCCODE;
Var countryPtr: Pointer;
    lconvInfo:  LCONV;
    sessionKey: NWDS_SESSION_KEY;
    DSCode:     int;
    DS:         Record
                cCode, cCode2: NWCCode
                end absolute DSCode;
begin
   AddTermination(srv);
   conn := 0;
   DS.ccode := NWAttachToFileServer(@srv[1], 0, conn);
   Case DS.cCode of
        0: begin  (* Set up locale information *)
(*         countryPtr := NWLsetlocale(LC_ALL, '');  { Initialize the LCONV structure to the locale information }
           NWLlocaleconv(lconvInfo);
           DSCode := NWInitUnicodeTables(lconvInfo.country_id, lconvInfo.code_page);
           if (DS.cCode=$FE15)  then DSCode := NWInitUnicodeTables(lconvInfo.country_id, 850); { file not found }
           if (DS.cCode=$FE15)  then DSCode := NWInitUnicodeTables(lconvInfo.country_id, 437);
           if (DS.cCode<>$FE17) then MyShowMessage('NWInitUnicodeTables returned '+hex4(DS.cCode)); { already loaded? }  *)
           DScode := NWDSAuthenticate(conn, 0, sessionKey);
           if (DS.cCode<>0)
              then MyShowMessage('NWDSAuthenticate returned '+hex4(DS.cCode))
              else begin
                   DS.cCode  := NWDSLockConnection(conn);
                   if (DS.cCode<>0)  then MyShowMessage('NWDSLockConnection returned '+hex4(DS.cCode));
{                  DScode := NWFreeUnicodeTables;
                   if (DS.cCode<>0)  then MyShowMessage('NWFreeUnicodeTables returned '+hex4(DS.cCode)); }
                   end;
           end;
        ALREADY_ATTACHED: DS.cCode := 0;
        else              MyShowMessage('NWAttachToFileServer ('+srv+') returned '+hex4(DS.cCode));
        end; { case }
   NDSattach := DS.cCode;
end;

Function NDSDetach(FS: String): NWCCODE;
VAR connID : NWCONN_HANDLE;
begin
   AddTermination(FS);
   cCode := NWGetConnectionIDFromName(Length(FS), @FS[1], connID);
   if (cCode=0) then begin
      cCode := NWDSUnlockConnection(connID);
      if (cCode=0) then cCode := NWFreeConnectionSlot(connID, SYSTEM_DISCONNECT);
      end;
   NDSDetach := cCode;
end;

PROCEDURE FSplit(Path: String; VAR D: DirStr; VAR N: NameStr; VAR E: ExtStr);
VAR Res : WORD;
    F   : FilePath;
BEGIN
  F.Dir  := ExtractFilePath(Path);
  F.Name := ExtractFileName(Path);
  Res := Pos('.', F.Name); if Res>0 then F.Name := Copy(F.Name, 1, Res-1); 
  F.Ext  := ExtractFileExt(Path);
  D      := F.Dir;
  N      := F.Name;
  E      := F.Ext;
END;

FUNCTION CopyFile(Source : String; Destination : String): NWCCODE;
{         Kopiert bel. Files;  Zielfilename optional; Pfadnamen optional
          Bsp.: CopyFile(A:\sub\xyz.exe, C:\UTIL\TOOLS, result)
                CopyFile(A:\sub\xyz.exe, C:\UTIL\TOOLS\x.exe, result)
          frei nach TP5 manual S. II-252 }
CONST BufSize = $1000;
TYPE
   TBuffer = ARRAY[1..BufSize] of Byte;  {16kB-Blcke werden kopiert}
   PBuffer = ^TBuffer;
VAR
   SF, DF           : FILE;
   SFN, DFN         : FilePath;
   IO               : BYTE;
   CopyOK           : NWCCODE;
   FSRes            : WORD;
   SrcHdl, DstHdl   : Integer;
   BytesRead,BytesWritten : LongInt;
   Puffer           : PBuffer;

BEGIN
FSplit(Source,      SFN.Dir,SFN.Name,SFN.Ext);
FSplit(Destination, DFN.Dir,DFN.Name,DFN.Ext);
IF DFN.Name='' THEN
   BEGIN
   DFN.Name := SFN.Name;
   DFN.Ext  := SFN.Ext;
   Destination := DFN.Dir+DFN.Name+DFN.Ext;
   END;
{ CopyOK := FileExists(Source); }
CopyOK := $FFFF;
SrcHdl := FileOpen(Source, OpenRead+DenyWrite);
if (SrcHdl>0) then begin
   DstHdl := FileCreate(Destination);
   if (DstHdl>0) then begin
      Puffer := New(PBuffer);
      if Puffer<>nil then begin
         repeat
            BytesRead    := FileRead(SrcHdl, Puffer^, BufSize);
            BytesWritten := FileWrite(DstHdl, Puffer^, BytesRead);
            until (BytesRead < BufSize);
         Dispose(Puffer);
         FileSetDate(DstHdl, FileGetDate(SrcHdl));
         if BytesRead=BytesWritten then CopyOK := 0;
         end;
      FileClose(DstHdl);
      end;
   FileClose(SrcHdl);
   FileSetAttr(Destination, FileGetAttr(Source));
   end;
   CopyFile := CopyOK;
END;


FUNCTION NCPCall(connID: NWCONN_HANDLE; NCPFct: Byte; ReqLen: WORD; VAR Req; RepLen: WORD; VAR Rep): NWCCODE;
VAR FragReq, FragRep : NW_FRAGMENT;
begin
   FragReq.fragAddress := @Req;
   FragReq.fragSize1   := ReqLen;
   FragRep.fragAddress := @Rep;
   FragRep.fragSize1   := RepLen;
   NCPCall := NWRequest(connID, NCPFct, 1, FragReq, 1, FragRep);
end;

FUNCTION ReturnBinderyContext(connID: NWCONN_HANDLE; VAR BContext: String): NWCCODE;
CONST MaxCtx = 200;
TYPE TReturnBinderyContextReq = Record
     SubFct  : Byte;
     end;
TYPE TReturnBinderyContextRep = Record
     Len     : DWORD;
     binderyContext : Array[0..2*MaxCtx-1] of Char;
     end;
VAR Req : TReturnBinderyContextReq;
    Rep : TReturnBinderyContextRep;
    i   : WORD;
    BC  : String;
begin
   Req.SubFct  := 4;
   ReturnBinderyContext := NCPCall(connID, 104,  SizeOf(Req), Req, SizeOf(Rep), Rep);
   for i := 1 to MaxCtx do BC[i] := Rep.BinderyContext[2*(i-1)];
   for i := MaxCtx DownTo 1 do if BC[i]=#0 then BC[0] := Char(i-1);
   BContext := BC
END;

FUNCTION ReturnTreeName(connID: NWCONN_HANDLE; VAR TreeName : String): NWCCODE;
CONST MaxTree = 255;
TYPE TPingForNDSReq = Record
     SubFct   : Byte;
     Reserved : Array[1..3] of Byte;
     end;
TYPE TPingForNDSRep = Record
     PingVersion : LongInt;
     TreeNameLen : LongInt;
     Tree        : Array[0..MaxTree] of Char;
     end;
VAR Req : TPingForNDSReq;
    Rep : TPingForNDSRep;
    i   : WORD;
    size: WORD;
    Temp  : String;
{ VAR cCode : NWCCODE; }
begin
   Req.SubFct  := 1;
   TreeName    := '';

   cCode := NCPCall(connID, 104,  SizeOf(Req), Req, SizeOf(Rep), Rep);
   if cCode=0 then begin
      size := Min(MaxTree, Rep.TreeNameLen);
      Move(Rep.Tree, TreeName[1], size);
      TreeName[0] := Char(size);
      While (Length(TreeName)>0) and (TreeName[Length(TreeName)]=#0) do Dec(TreeName[0]); 
      end;
   ReturnTreeName := cCode;
END;

FUNCTION ReloadDSSoftware(connID: NWCONN_HANDLE; VAR Status: int4): NWCCODE;
TYPE TReloadDSSoftwareReq = Record
     SubFct   : Byte;
     end;
TYPE TReloadDSSoftwareRep = Record
     Status      : int4;
     Flags       : int4;
     end;
VAR Req : TReloadDSSoftwareReq;
    Rep : TReloadDSSoftwareRep;
begin
   Req.SubFct  := 8;
   ReloadDSSoftware := NCPCall(connID, 104,  SizeOf(Req), Req, SizeOf(Rep), Rep);
   Status := Rep.Status;
END;


Function NW40_LoadAnNLM(connID: NWCONN_HANDLE; VAR RPCCode: LongInt; NLMLoadOptions: LongInt; PathAndName: String): NWCCODE;
VAR Req : Record
    FctLen         : WORD;
    SubFct         : Byte;
    NLMLoadOptions : LongInt;
    reserved       : Array[1..3] of LongInt;
    reservedFlags  : Array[1..4] of Byte;
    PathAndName    : Array[0..255] of Char;
    END;
VAR Rep : Record
    RPCCode        : LongInt;
    Reserved       : Array[1..4] of LongInt;
    end;
BEGIN
   FillChar(Req, SizeOf(Req), 0);
   PathAndName         := PathAndName+#0;
   Req.FctLen          := Length(PathAndName)+30;
   Req.NLMLoadOptions  := NLMLoadOptions;
   Move(PathAndName[1], Req.PathAndName, Length(PathAndName)+1);
   Req.SubFct     := 1;
   NW40_LoadAnNLM := NCPCall(connID, 131,  Req.FctLen, Req, SizeOf(Rep), Rep);
   RPCCode := Rep.RPCCode;
END;

Function NW40_UnloadAnNLM(connID: NWCONN_HANDLE; VAR RPCCode: LongInt; PathAndName: String): NWCCODE;
VAR Req : Record
    FctLen         : WORD;
    SubFct         : Byte;
    reserved       : Array[1..4] of LongInt;
    reservedFlags  : Array[1..4] of Byte;
    PathAndName    : Array[0..255] of Char;
    END;
VAR Rep : Record
    RPCCode        : LongInt;
    Reserved       : Array[1..4] of LongInt;
    end;
BEGIN
   FillChar(Req, SizeOf(Req), 0);
   PathAndName         := PathAndName+#0;
   Req.FctLen          := Length(PathAndName)+30;
   Move(PathAndName[1], Req.PathAndName, Length(PathAndName)+1);
   Req.SubFct     := 2;
   NW40_UnloadAnNLM := NCPCall(connID, 131,  Req.FctLen, Req, SizeOf(Rep), Rep);
   RPCCode := Rep.RPCCode;
END;

Function NW40_MountVolume(connID: NWCONN_HANDLE; VAR RPCCode: LongInt; VolumeName: String): NWCCODE;
{ Volume Name without ':' }
VAR Req : Record
    FctLen         : WORD;
    SubFct         : Byte;
    reserved       : Array[1..4] of LongInt;
    reservedFlags  : Array[1..4] of Byte;
    VolumeName     : Array[0..255] of Char;
    END;
VAR Rep : Record
    RPCCode        : LongInt;
    Reserved       : Array[1..4] of LongInt;
    VolumeNumber   : LongInt;
    end;
BEGIN
   FillChar(Req, SizeOf(Req), 0);
   VolumeName          := VolumeName+#0;
   Req.FctLen          := Length(VolumeName)+30;
   Move(VolumeName[1], Req.VolumeName, Length(VolumeName)+1);
   Req.SubFct  := 3;
   NW40_MountVolume := NCPCall(connID, 131,  Req.FctLen, Req, SizeOf(Rep), Rep);
   RPCCode := Rep.RPCCode;
END;

Function NW40_DismountVolume(connID: NWCONN_HANDLE; VAR RPCCode: LongInt; VolumeName: String): NWCCODE;
{ Volume Name without ':' }
VAR Req : Record
    FctLen         : WORD;
    SubFct         : Byte;
    reserved       : Array[1..4] of LongInt;
    reservedFlags  : Array[1..4] of Byte;
    VolumeName     : Array[0..255] of Char;
    END;
VAR Rep : Record
    RPCCode        : LongInt;
    Reserved       : Array[1..4] of LongInt;
    end;
BEGIN
   FillChar(Req, SizeOf(Req), 0);
   VolumeName          := VolumeName+#0;
   Req.FctLen          := Length(VolumeName)+30;
   Move(VolumeName[1], Req.VolumeName, Length(VolumeName)+1);
   Req.SubFct  := 4;
   NW40_DismountVolume := NCPCall(connID, 131,  Req.FctLen, Req, SizeOf(Rep), Rep);
   RPCCode := Rep.RPCCode;
END;

Function NW40_AddNameSpaceToVolume(connID:NWCONN_HANDLE; VAR RPCCode:LongInt; VolumeName:String; NameSpaceName:String):NWCCODE;
VAR NS : String;
VAR Req : Record  {'namespace' must contain:  "<namespc> [to] [volume] <volume>\0"}
    FctLen         : WORD;
    SubFct         : Byte;
    reserved       : Array[1..4] of LongInt;
    reservedFlags  : Array[1..4] of Byte;
    NameSpace      : Array[0..255] of Char;
    END;
VAR Rep : Record
    RPCCode        : LongInt;
    Reserved       : Array[1..4] of LongInt;
    end;
BEGIN
   FillChar(Req, SizeOf(Req), 0);
   NS                  := VolumeName + ' ' + NameSpaceName +#0;
   Req.FctLen          := Length(NS)+30;
   Move(NS[1], Req.NameSpace , Length(NS)+1);
   Req.SubFct  := 5;
   NW40_AddNameSpaceToVolume := NCPCall(connID, 131,  Req.FctLen, Req, SizeOf(Rep), Rep);
   RPCCode := Rep.RPCCode;
END;

Function NW40_ExecuteNCFFile(connID: NWCONN_HANDLE; VAR RPCCode: LongInt; PathAndName: String): NWCCODE;
VAR Req : Record
    FctLen         : WORD;
    SubFct         : Byte;
    reserved       : Array[1..4] of LongInt;
    reservedFlags  : Array[1..4] of Byte;
    PathAndName     : Array[0..255] of Char;
    END;
VAR Rep : Record
    RPCCode        : LongInt;
    Reserved       : Array[1..4] of LongInt;
    end;
BEGIN
   FillChar(Req, SizeOf(Req), 0);
   PathAndName          := PathAndName+#0;
   Req.FctLen           := Length(PathAndName)+30;
   Move(PathAndName[1], Req.PathAndName, Length(PathAndName)+1);
   Req.SubFct  := 7;
   NW40_ExecuteNCFFile := NCPCall(connID, 131,  Req.FctLen, Req, SizeOf(Rep), Rep);
   RPCCode := Rep.RPCCode;
END;

Function NW40_SetSetCommandValue(connID: NWCONN_HANDLE; VAR RPCCode: LongInt; SetCommandName: String;
                                 LValue: LongInt; SValue: String): NWCCODE;
VAR Req : Record
    FctLen         : WORD;
    SubFct         : Byte;
    typeFlag       : LongInt;
    LValue         : LongInt;
    reserved       : Array[1..2] of LongInt;
    reservedFlags  : Array[1..4] of Byte;
    SetCommandName : Array[0..255] of Char;
    END;
    L1, L2: WORD;
VAR Rep : Record
    RPCCode        : LongInt;
    Reserved       : Array[1..4] of LongInt;
    end;
BEGIN
   SetCommandName:= SetCommandName+#0;
   L1 := Length(SetCommandName);
   Move(SetCommandName[1], Req.SetCommandName, L1);
   if LValue>=0 then begin  {use numeric value;   also 0 for'Off', 1 for 'Off' }
      Req.typeFlag := 1;
      Req.LValue   := LValue;
      end
     else begin             {use string value}
      Req.typeFlag := 0;
      Req.LValue   := 0;
      SValue       := SValue+#0;
      L2           := Length(SValue);
      Move(SValue[1], Req.SetCommandName[L1], L2);
      end;
  Req.FctLen  := Length(SetCommandName)+30;
  Req.SubFct  := 6;
   NW40_SetSetCommandValue := NCPCall(connID, 131,  Req.FctLen, Req, SizeOf(Rep), Rep);
   RPCCode := Rep.RPCCode;
END;




FUNCTION NWCallsInit;                       external 'NWCALLS';

Function NWAddObjectToSet;                  external 'NWCALLS';
FUNCTION NWAddTrustee;                      external 'NWCALLS';
FUNCTION NWAddTrusteeToDirectory;           external 'NWCALLS';
FUNCTION NWAttachToFileServer;              external 'NWCALLS';
FUNCTION NWBroadcastToConsole;              external 'NWCALLS';
FUNCTION NWChangeObjectPassword;            external 'NWCALLS' index 18;
FUNCTION NWChangeObjectSecurity;            external 'NWCALLS';
FUNCTION NWCheckConsolePrivileges;          external 'NWCALLS';
FUNCTION NWCheckNetWareVersion;             external 'NWCALLS';
Function NWClearPhysicalRecord;             external 'NWCALLS';
Function NWCreateDirectory;                 external 'NWCALLS';
FUNCTION NWCreateObject;                    external 'NWCALLS';
FUNCTION NWCreateProperty;                  external 'NWCALLS';
FUNCTION NWCreateQueue;                     external 'NWCALLS';
Function NWDeleteObject;                    external 'NWCALLS';
FUNCTION NWDeleteObjectFromSet;             external 'NWCALLS';
FUNCTION NWDeleteProperty;                  external 'NWCALLS';
FUNCTION NWDestroyQueue;                    external 'NWCALLS';
FUNCTION NWDetachFromFileServer;            external 'NWCALLS';
FUNCTION NWDisableFileServerLogin;          external 'NWCALLS';
FUNCTION NWDisableTTS;                      external 'NWCALLS';
FUNCTION NWDownFileServer;                  external 'NWCALLS';

FUNCTION NWDSAuthenticate;                  external 'NWNET';
FUNCTION NWDSGetConnectionSlot;             external 'NWNET';
FUNCTION NWDSLockConnection;                external 'NWNET';
FUNCTION NWDSUnlockConnection;              external 'NWNET';

FUNCTION NWEnableFileServerLogin;           external 'NWCALLS';
FUNCTION NWEnableTTS;                       external 'NWCALLS';
FUNCTION NWFreeConnectionSlot;              external 'NWNET';
FUNCTION NWFreeUnicodeTables;               external 'NWLOCALE';
FUNCTION NWGetActiveLANBoardList;           external 'NWCALLS';
FUNCTION NWGetBinderyAccessLevel;           external 'NWCALLS';
FUNCTION NWGetCacheInfo;                    external 'NWCALLS';
FUNCTION NWGetConnectionHandle;             external 'NWCALLS';
FUNCTION NWGetConnectionIDFromName;         external 'NWNET';
FUNCTION NWGetConnectionInformation;        external 'NWCALLS';
FUNCTION NWGetConnectionStatus;             external 'NWCALLS';
FUNCTION NWGetConnectionList;               external 'NWCALLS';
FUNCTION NWGetConnectionNumber;             external 'NWCALLS';
FUNCTION NWGetDefaultConnectionID;          external 'NWCALLS';
FUNCTION NWGetDiskCacheStats;               external 'NWCALLS';
FUNCTION NWGetExtendedVolumeInfo;           external 'NWCALLS';
FUNCTION NWGetFileServerInfo;               external 'NWCALLS';
FUNCTION NWGetFileServerInformation;        external 'NWCALLS';
FUNCTION NWGetFileServerLoginStatus;        external 'NWCALLS';
FUNCTION NWGetFSLANDriverConfigInfo;        external 'NWCALLS';
FUNCTION NWGetLANConfigInfo;                external 'NWCALLS';
FUNCTION NWGetNearestDirectoryService;      external 'NWNET';
FUNCTION NWGetNetworkSerialNumber;          external 'NWCALLS';
FUNCTION NWGetNLMInfo;                      external 'NWCALLS';
FUNCTION NWGetNLMLoadedList;                external 'NWCALLS';
FUNCTION NWGetNumConnections;               external 'NWNET';
FUNCTION NWGetObjectConnectionNumbers;      external 'NWCALLS';
Function NWGetObjectID;                     external 'NWCALLS';
FUNCTION NWGetObjectName;                   external 'NWCALLS';
FUNCTION NWGetOSVersionInfo;                external 'NWCALLS';
Function NWGetPrimaryConnectionID;          external 'NWCALLS';
Function NWGetServerSetCommandsInfo;        external 'NWCALLS';
Function NWGetServerSetCategories;          external 'NWCALLS';
FUNCTION NWGetVolumeInfoWithNumber;         external 'NWCALLS';
FUNCTION NWInitUnicodeTables;               external 'NWLOCALE';
FUNCTION NWIsManager;                       external 'NWCALLS';
FUNCTION NWLlocaleconv;                     external 'NWLOCALE';
FUNCTION NWLoginToFileServer;               external 'NWCALLS' index 16;
Function NWLogPhysicalRecord;               external 'NWCALLS';
Function NWLsetlocale;                      external 'NWLOCALE';
FUNCTION NWReadPropertyValue;               external 'NWCALLS';
Function NWRenameObject;                    external 'NWCALLS';
FUNCTION NWRequest;                         external 'NWCALLS';
FUNCTION NWScanObject;                      external 'NWCALLS';
Function NWScanObjectTrusteePaths;          external 'NWCALLS';
Function NWScanProperty;                    external 'NWCALLS';
{Function NWSetLockMode;                    external 'NWCALLS';}
FUNCTION NWSetPreferredServer;              external 'NWCALLS';
FUNCTION NWTTSIsAvailable;                  external 'NWCALLS';

FUNCTION NWVerifyObjectPassword;            external 'NWCALLS' index 17;
Function NWWritePropertyValue;              external 'NWCALLS';



BEGIN
END.
