unit XferMain;

{ History for XFer:
  95-09-25:   Uploaded to NovUser
  95-09-28:   added NWDestroyQueue
  95-10-31:   added 'SECURITY_EQUALS' for clone/copy group
}

interface

uses
  NWCalls,
  XferObj, XferOpt, XFerHelp, XferUnit, Delphi_U,
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Menus, ExtCtrls;

type

  TMainScreen = class(TForm)
    MainMenu1: TMainMenu;
    BinderyMenu: TMenuItem;
    ScanBindery1: TMenuItem;
    RadioGroupObjects: TRadioGroup;
    Exit1: TMenuItem;
    Exit: TButton;
    RadioGroupSrv0: TRadioGroup;
    RadioGroupSrv1: TRadioGroup;

    ListBox0: TListBox;
    ListBox1: TListBox;

    CopyButton: TButton;
    RenButton: TButton;
    EditRename: TEdit;
    ShowButton: TButton;
    CloneButton: TButton;
    AddButton: TButton;
    DelButton: TButton;
    Label1: TLabel;
    Label2: TLabel;

    Edit1: TMenuItem;
    Rename1: TMenuItem;
    Delete1: TMenuItem;
    Show1: TMenuItem;
    Copy1: TMenuItem;
    CopyObj1: TMenuItem;
    Configure1: TMenuItem;
    EditDefaults1: TMenuItem;
    ShowSecurityEquivalences1: TMenuItem;
    ShowTrustees1: TMenuItem;
    ShowGroupsMembers1: TMenuItem;
    N1: TMenuItem;
    ButtonCfg: TButton;
    LabelSearchPattern: TLabel;
    EditDefSearchObj: TEdit;
    Clone1: TMenuItem;
    Add1: TMenuItem;
    ButtonTest: TButton;
    Help1: TMenuItem;
    GetErrorList1: TMenuItem;
    About1: TMenuItem;
    procedure CheckAttachments;
    procedure initialize;

    Procedure DisableMenuItems;
    procedure StatusInfo(Pos: Integer; Text: String);
    procedure DelListEntry(Sender: TObject; SelectedListbox: TListBox; Pos: Integer);
    procedure SelectEntryFromMainList(Pos: Integer; srvNr: BYTE);

    procedure AddButtonClick(Sender: TObject);
    procedure DelButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ScanBinderyClick(Sender: TObject; srvNr: Word);
    procedure ScanBindery0Click(Sender: TObject);
    procedure ScanBindery1Click(Sender: TObject);
    procedure ListBoxSingleClick0(Sender: TObject);
    procedure ListBoxSingleClick1(Sender: TObject);
    procedure ListBoxDoubleClick(Sender: TObject; srvNr: Byte);
    procedure ListBoxDoubleClick0(Sender: TObject);
    procedure ListBoxDoubleClick1(Sender: TObject);
    procedure ItemSelected(Sender: TObject; VAR pos: Integer);
    procedure ResetOneList(Sender: TObject; srvNr: BYTE);
    procedure ResetListClick(Sender: TObject);
    procedure LookIntoClick(Sender: TObject);
    procedure DefineSearchObject(Sender: TObject);
    procedure RadioGroupObjectsClick(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure SelectServer(Sender: TObject; srvNr: WORD; idxNr: WORD);
    procedure RadioGroupSrv0Click(Sender: TObject);
    procedure RadioGroupSrv1Click(Sender: TObject);
    procedure CopyOrCloneObject(Sender: TObject; sourceSrv, dstSrv: Byte);
    procedure CopyButtonClick(Sender: TObject);
    procedure GetNameFromEditBox(Sender: TObject; VAR Name: String);
    procedure RenButtonClick(Sender: TObject);
    procedure EditRenameChange(Sender: TObject);

    procedure EditAddChange(Sender: TObject);
    procedure RenameObject(Sender: TObject; VAR O: scanObjectRep; VAR NewName: String; SelectedListbox: TListBox);

    procedure ShowButtonClick(Sender: TObject);
    procedure EditDefaults1Click(Sender: TObject);
    procedure ShowObjectDetails(Sender: TObject; PageName: String);
    procedure ShowObjectClick(Sender: TObject);
    procedure ShowTrustees1Click(Sender: TObject);
    procedure ShowGroupsMembers1Click(Sender: TObject);
    procedure ShowSecurityEquivalences1Click(Sender: TObject);
    procedure CloneButtonClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure EditDefSearchObjChange(Sender: TObject);
    procedure EditDefSearchObjExit(Sender: TObject);
    procedure GetErrorList1Click(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

var
  MainScreen: TMainScreen;

implementation

{$R *.DFM}

procedure TMainScreen.CheckAttachments;
VAR connInfo:       CONNECT_INFO;
    conn:           BYTE;
    srvName:        Str48;
    connectionList: TNWConnHandleList;
begin
     cCode         := NWGetNumConnections(numConnections); {max configured}
                      if cCode<>0 THEN numConnections := 8;
                      if numConnections>19 THEN numConnections := 19;
     cCode         := NWGetConnectionList(0, connectionList, numConnections, numConnections); {max conn used}
     for conn := 0 to numConnections do begin
         cCode     := NWGetConnectionStatus(connectionList[conn], connInfo, sizeOf(connInfo));
         if cCode=0 THEN BEGIN
            srvName := StrPas(@connInfo.serverName[1]);
            RadioGroupSrv0.Items.Add(srvName);
            RadioGroupSrv1.Items.Add('');
            END;
         end;
     RadioGroupSrv0.ItemIndex := 0;
     if numConnections>1 THEN RadioGroupSrv1.ItemIndex := 1
                         else RadioGroupSrv1.ItemIndex := 0;
end;

procedure TMainScreen.initialize;
begin
     cCode         := NWCallsInit(NIL, NIL);
     cCode         := NWGetDefaultConnectionID(connID[0]);
     connID[1]     := connID[0];
end;

Procedure TMainScreen.DisableMenuItems;
begin
   Delete1.Enabled := DelButton.Enabled;
   CopyObj1.Enabled:= CopyButton.Enabled;
   Rename1.Enabled := RenButton.Enabled;
   Clone1.Enabled  := CloneButton.Enabled;
   Add1.Enabled    := AddButton.Enabled;
   Show1.Enabled   := ShowButton.Enabled;
   ShowSecurityEquivalences1.Enabled := Show1.Enabled;
   ShowTrustees1.Enabled             := Show1.Enabled;
   ShowGroupsMembers1.Enabled        := Show1.Enabled;
end;

procedure TMainScreen.StatusInfo(Pos: Integer; Text: String);
begin   { if pos=-1, list is empty }
     if (Pos>=0) then EditRename.Text    :=  Text;
     RenButton.Enabled   := (FALSE);
     AddButton.Enabled   := (FALSE);
     CloneButton.Enabled := (FALSE);
     DelButton.Enabled   := (Pos>=0);
     ShowButton.Enabled  := (Pos>=0);
     CopyButton.Enabled  := (Pos>=0);

     Label1.Caption := IntToStr(ListBox0.Items.Count) + ' Items';
     Label2.Caption := IntToStr(ListBox1.Items.Count) + ' Items';

     if lastSrv = 0 THEN CopyButton.Caption := '>> Copy >>'
                    else CopyButton.Caption := '<< Copy <<';
     DisableMenuItems;
end;

procedure TMainScreen.AddButtonClick(Sender: TObject);
VAR NewPos: Integer;
begin
   NotYet(Sender);
   ShowMessage('"Clone" Existing Objects Instead');
end;

procedure TMainScreen.DelListEntry(Sender: TObject; SelectedListbox: TListBox; Pos: Integer);
begin
   SelectedListbox.Items.Delete(Pos);
   SelectEntryFromMainList(Pos, lastSrv);
end;

procedure TMainScreen.DelButtonClick(Sender: TObject);
VAR Pos:    Integer;
    Name:   String;
    O:      scanObjectRep;
begin
     if (searchObjType <> $FFFF) THEN begin{do not del on wildcard search}
        if lastSrv=0 THEN SelectedListbox    := ListBox0
                     else SelectedListbox    := ListBox1;
        Pos   := SelectedListbox.ItemIndex;
        Name  := SelectedListbox.Items[Pos];
        AddTermination(Name);

        if ((Not ConfigDialog.CheckWarnDel.Checked) or (AreYouSure = mrOK)) then BEGIN
           if searchObjType <> Swap(OT_PRINT_QUEUE)
              then cCode := NWDeleteObject(connID[lastSrv], @Name[1], searchObjType)
              else Begin
                   GetObjectInfo(Sender, searchObjType, @Name[1], connID[lastSrv], O);
                   cCode := NWDestroyQueue(connID[lastSrv], O.ObjectID);
                   END;
           ShowError(cCode, 'DeleteObject', Err, CFGErrorThreshold);
           if cCode=0 then BEGIN
              DeleteListEntry(Sender, SelectedListbox, Pos);
              SelectEntryFromMainList(Pos, lastSrv);
              END;
           end;
        end;
end;

procedure TMainScreen.FormCreate(Sender: TObject);
begin
     initialize;
     EditRename.Text    := '';
end;

procedure TMainScreen.ScanBinderyClick(Sender: TObject; srvNr: Word);
VAR
      SelectedRadioGroup:TRadioGroup;
begin
     connID[srvNr] := srvInfo[srvNr].connID;
     if srvNr=0 THEN SelectedListbox    := ListBox0
                else SelectedListbox    := ListBox1;
     if srvNr=0 THEN SelectedRadioGroup := RadioGroupSrv0
                else SelectedRadioGroup := RadioGroupSrv1;
     ListBoxAddObjects(Sender, connID[srvNr], SelectedListbox, @CFGSearchPattern[1], searchObjType);
end;


procedure TMainScreen.ScanBindery0Click(Sender: TObject);
begin
     ScanBinderyClick(Sender, 0);
end;

procedure TMainScreen.ScanBindery1Click(Sender: TObject);
begin
     ScanBinderyClick(Sender, 1);
end;

procedure TMainScreen.ListBoxSingleClick0(Sender: TObject);
begin
     SelectEntryFromMainList(ListBox0.ItemIndex, 0);
end;

procedure TMainScreen.ListBoxSingleClick1(Sender: TObject);
begin
     SelectEntryFromMainList(ListBox1.ItemIndex, 1);
end;

procedure TMainScreen.ListBoxDoubleClick(Sender: TObject; srvNr: Byte);
VAR O:                 scanObjectRep;
begin
     lastSrv                := srvNr;
     if srvNr=0 THEN SelectedListbox    := ListBox0
                else SelectedListbox    := ListBox1;

     GetCurrentObject(Sender, SelectedListbox, O);
     IF cCode = 0 THEN BEGIN
        usrInfo[srvNr].objectID   := O.ObjectID;
        usrInfo[srvNr].ObjectName := O.ObjectName;
        usrInfo[srvNr].ObjectType := O.ObjectType;
        MultPageDlg.FillFirstPage(Sender, O, SelectedListBox);
        END;
end;

procedure TMainScreen.ListBoxDoubleClick0(Sender: TObject);
begin
   ListBoxDoubleClick(Sender, 0);
end;

procedure TMainScreen.ListBoxDoubleClick1(Sender: TObject);
begin
   ListBoxDoubleClick(Sender, 1);
end;

procedure TMainScreen.ShowButtonClick(Sender: TObject);
begin
   ListBoxDoubleClick(Sender, lastSrv);
end;

procedure TMainScreen.ItemSelected(Sender: TObject; VAR pos: Integer);
begin
   if lastSrv=0 THEN SelectedListbox    := ListBox0
                else SelectedListbox    := ListBox1;
   Pos := SelectedListbox.ItemIndex;
end;

procedure TMainScreen.SelectEntryFromMainList(Pos: Integer; srvNr: Byte);
VAR   NotSelectedListbox: TListBox;
begin
     if srvNr=0
        THEN BEGIN SelectedListbox := ListBox0; NotSelectedListbox := ListBox1; END
        else BEGIN SelectedListbox := ListBox1; NotSelectedListbox := ListBox0; END;

     if lastSrv <> srvNr THEN BEGIN        { unmark items in other listbox }
{       NotSelectedListbox.MultiSelect  := (FALSE); }
        NotSelectedListbox.ItemIndex    := -1;
{       NotSelectedListbox.MultiSelect  := (TRUE); }
        lastSrv                         := srvNr;
        end;

     IF Pos <> SelectedListbox.ItemIndex THEN BEGIN {change current index}
        IF Pos<0 THEN Pos := 0  else
           IF Pos>=SelectedListbox.Items.Count THEN
              Pos := Pred(SelectedListbox.Items.Count);
        END;

     if srvNr=0 THEN ListBox0.ItemIndex := Pos
                else ListBox1.ItemIndex := Pos;
     curObj[srvNr]                      := Pos;

     IF SelectedListbox.Items.Count>0
        THEN StatusInfo(Pos, SelectedListbox.Items[Pos])
        else StatusInfo(-1, '');
end;

procedure TMainScreen.ResetOneList(Sender: TObject; srvNr: BYTE);
begin
     if srvNr=0 THEN ClearList(ListBox0)
                else ClearList(ListBox1);
     ScanBinderyClick(Sender, srvNr);
     StatusInfo(-1, '');
end;

procedure TMainScreen.ResetListClick(Sender: TObject);
begin
   ClearList(ListBox0);
   ClearList(ListBox1);
   ResetOneList(Sender, 0);
   ResetOneList(Sender, 1);
end;

procedure TMainScreen.LookIntoClick(Sender: TObject);
begin
   MultPageDlg.ShowModal;
end;


procedure TMainScreen.DefineSearchObject(Sender: TObject);
begin
   Case RadioGroupObjects.ItemIndex of
     1..4: searchObjType := Swap(RadioGroupObjects.ItemIndex);
     5:    searchObjType := Swap(OT_PRINT_SERVER);
     0:    searchObjType := $FFFF;
     END; {case}
end;

procedure TMainScreen.RadioGroupObjectsClick(Sender: TObject);
begin
   DefineSearchObject(Sender);
   ResetListClick(Sender);
end;


procedure TMainScreen.Exit1Click(Sender: TObject);
begin
    Close;
end;

procedure TMainScreen.SelectServer(Sender: TObject; srvNr: WORD; idxNr: WORD);
VAR srvName:            String;
    CONN_ID:            NWCONN_HANDLE;
begin
   srvName := RadioGroupSrv0.Items[idxNr];
   AddTermination(srvName);
   cCode   := NWGetConnectionHandle(@srvName[1], 0, CONN_ID, NIL);
   if cCode=0 THEN BEGIN
      cCode := NWGetConnectionStatus(CONN_ID, srvInfo[srvNr], sizeOf(CONNECT_INFO));
      connID[srvNr] := CONN_ID;
      NWVers[srvNr] := NW_Version(connID[srvNr]);
      end;
   ResetOneList(Sender, srvNr);
end;

procedure TMainScreen.RadioGroupSrv0Click(Sender: TObject);
begin
   SelectServer(Sender, 0, RadioGroupSrv0.ItemIndex);
end;

procedure TMainScreen.RadioGroupSrv1Click(Sender: TObject);
begin
   SelectServer(Sender, 1, RadioGroupSrv1.ItemIndex);
end;

procedure TMainScreen.CopyOrCloneObject(Sender: TObject; sourceSrv, dstSrv: Byte);
VAR
    O1, O2:            scanObjectRep;
    TargetListbox:     TListBox;
    Pos:               WORD;
    Name, NewName:     String;
    Prop:              scanPropRep;
    cCodeP,cCodeV:     NWCCODE;
    cCodeM,cCodeN:     NWCCODE;
    i:                 byte;
    MmbArray:          Array[0..31] of NWOBJ_ID;
    Mmb:               TBinderyObject;
    OldSwitchOverwrite:Boolean;
    accessLevel:       NWFLAGS;
    objectID:          NWOBJ_ID;
    CanGoOn:           Boolean;
    Prop2:             PChar;
    PWD:               String;
begin
     Screen.Cursor := crHourGlass;
     OldSwitchOverwrite := ConfigDialog.CopyOverwrite.Checked;
     CanGoOn := True;

     if (not ConfigDialog.CopyPartial.Checked) then begin
        cCode := NWGetBinderyAccessLevel(connID[dstSrv], accessLevel, objectID);
        if (accessLevel < $33) then begin
           ShowError($8000+accessLevel, 'Insufficient Rights on Target', Err, CFGErrorThreshold);
           CanGoOn := False;
           end
          else
           begin
           cCode := NWGetBinderyAccessLevel(connID[sourceSrv], accessLevel, objectID);
           if (accessLevel < $33) then begin
              ShowError($8000+accessLevel, 'Insufficient Rights on Source', Err, CFGErrorThreshold);
              CanGoOn := False;
              end;
           end;
        end;

     if sourceSrv=0  THEN SelectedListbox    := ListBox0
                     else SelectedListbox    := ListBox1;
     if dstSrv=0     THEN TargetListbox      := ListBox0
                     else TargetListbox      := ListBox1;

     if ((CanGoOn) and (SelectedListBox.Items.Count>0)) THEN begin
        { Clone Object }
        Name := SelectedListBox.Items[SelectedListBox.ItemIndex];
        AddTermination(Name);
        O1.ObjectID := -1;
        GetObjectInfo(Sender, searchObjType, @Name[1], connID[sourceSrv], O1);
        ShowError(cCode, 'GetObjectInfo', Err, CFGErrorThreshold);
        if cCode=0 then begin
           O2 := O1;

           GetNameFromEditBox(Sender, NewName);
           if (NewName<>'') then Move(NewName[1], O2.objectName, Length(NewName)+1); { incl #0 }

           cCode := NWGetObjectID(connID[dstSrv], @O2.objectName, O2.objectType, O2.objectID);  { check existence or target}
           if (cCode<>0) THEN begin
              if (Swap(O2.objectType) = OT_PRINT_QUEUE)
                 then BEGIN
                      cCode := NWCreateQueue(connID[dstSrv], @O2.ObjectName, O2.ObjectType, 0, 'SYS:SYSTEM', O2.objectID);
                      if (cCode = 0) then ConfigDialog.CopyOverwrite.Checked := TRUE;
                      END
                 else cCode := NWCreateObject(connID[dstSrv], @O2.ObjectName, O2.ObjectType, O2.ObjectFlag, O2.ObjectSecurity);
              if (cCode=0)       then TargetListbox.Items.Add(StrPas(@O2.ObjectName))      else
              if (ccode = $89EE) then ShowError(cCode, 'Object already existed', Warning, CFGErrorThreshold)
                                 else ShowError(cCode, 'CreateObject',           Err,     CFGErrorThreshold);
              cCode := NWGetObjectID(connID[dstSrv], @O2.objectName, O2.objectType, O2.objectID);  { get ID }
              if ((cCode=0) and (Swap(O2.ObjectType) in [OT_USER, OT_PRINT_SERVER])) then
                 begin  { prepare new objects - only created ones }
                 if CFGDefPWD = '&username' then PWD := NewName        else
{                if CFGDefPWD = '&ask'      then PWD := 'ASK'          else }
                                                 PWD := CFGDefPWD;
                 AddTermination(PWD);
                 cCode := NWChangeObjectPassword(connID[dstSrv], @O2.ObjectName, O2.ObjectType, '', @PWD[1]); { Create Pwd }
                 cCode := CreateMailBoxFromID(connID[dstSrv], O2.objectID, O2.objectType, CFGErrorThreshold);
                 end;
              end;

           if (ConfigDialog.CopyTrustees.Checked) then
              CopyTrustees(connID[sourceSrv], connID[dstSrv], O1.objectID, O2.objectID, NWVers[sourceSrv], NWVers[dstSrv]);

           { Scan and add properties }
           Prop.sequenceNumber := -1;
           repeat
              cCodeP:= NWScanProperty(connID[sourceSrv], @O1.ObjectName, O1.ObjectType, '*',
                       Prop.sequenceNumber, Prop.propertyName, Prop.propertyFlags,
                       Prop.propertySecurity, Prop.hasValueFlag, Prop.moreFlag);
                       if cCodeP<>$89FB THEN ShowError(cCodeP, 'NWScanProperty '+Prop.propertyName, Err, CFGErrorThreshold);

              if (cCodeP>0) then continue;

              if ((ASCIIz(Prop.propertyName) = 'Q_DIRECTORY') AND  { this will be created by NWCreateQueue }
                  (Swap(O1.ObjectType) = OT_PRINT_QUEUE))               then continue                  else
              if  (ASCIIz(Prop.propertyName) = 'GROUPS_I''M_IN') then Prop2 := 'GROUP_MEMBERS'  else
              if  (ASCIIz(Prop.propertyName) = 'GROUP_MEMBERS')  then Prop2 := 'GROUPS_I''M_IN' else
                                                                      Prop2 := '';

              if ((cCodeP=0) AND ((Prop.propertyFlags AND BF_Dynamic)=0)) then begin
                 { Add properties }
                 cCode := NWCreateProperty(connID[dstSrv], @O2.ObjectName, O2.ObjectType,
                          @Prop.propertyName, Prop.propertyFlags, Prop.propertySecurity);
                 ShowError(cCode, 'NWCreateProperty '+Prop.propertyName, Warning, CFGErrorThreshold);

                 case cCode of
                    0:     ;
                    $89ED: if (Not ConfigDialog.CopyOverwrite.Checked) then continue;  {Prop exists; don't overwrite values }
                    else   continue;
                    end; {case}

                 if (Prop.hasValueFlag>0) then begin    { Read properties }
                    PV.dataSetIndex := 1;
                    repeat
                    cCodeV := NWReadPropertyValue(connID[sourceSrv], @O1.ObjectName, O1.ObjectType,
                              @Prop.propertyName, PV.dataSetIndex, PV.data, PV.moreFlag, PV.propertyFlags);
                    case cCodeV of
                       $89EC: ShowError(cCodeV, 'NWReadProperty '+Prop.propertyName, Err, CFGErrorThreshold);
                       0:     begin
                       { Write property }
                       IF ((PV.propertyFlags AND BF_Set)=0) THEN BEGIN   {No Set}
                          { Write item property }
                          cCode := NWWritePropertyValue(connID[dstSrv], @O2.ObjectName, O2.ObjectType,
                                   @Prop.propertyName, PV.dataSetIndex, PV.data, PV.moreFlag);
                                   ShowError(cCode, 'NWWriteProperty '+Prop.propertyName, Err, CFGErrorThreshold);
                          end
                                else
                          begin
                          { Write set property }
                          Move(PV.data, MmbArray, SizeOf(MmbArray));
                          for i := 0 to 31 do
                              if MmbArray[i]<>0 THEN begin
                              { Read set members }
                              cCodeM := NWGetObjectName(connID[sourceSrv], MmbArray[i], Mmb.ObjectName, Mmb.ObjectType);
                              ShowError(cCodeM, 'NWGetMmbName '+Mmb.ObjectName, Err, CFGErrorThreshold);
                              if cCodeM=0 then begin
                                 cCode := NWAddObjectToSet(connID[dstSrv], @O2.ObjectName, O2.ObjectType,
                                          @Prop.propertyName, @Mmb.ObjectName, Mmb.ObjectType);
                                 if ((cCode=$89E9) or (cCode=$89FC))
                                    THEN ShowError(cCode, 'NWAddMmbName '+StrPas(@Mmb.ObjectName), Warning, CFGErrorThreshold)
                                    else ShowError(cCode, 'NWAddMmbName '+StrPas(@Mmb.ObjectName), Err,     CFGErrorThreshold);

                                 if (StrLen(Prop2)>0) then begin
                                    cCode := NWAddObjectToSet(connID[dstSrv], @Mmb.ObjectName, Mmb.ObjectType,
                                             Prop2, @O2.ObjectName, O2.ObjectType);
                                    if ((cCode=$89E9) or (cCode=$89FC))
                                       THEN ShowError(cCode, 'NWAddMmb '+StrPas(@Mmb.ObjectName), Warning, CFGErrorThreshold)
                                       else ShowError(cCode, 'NWAddMmb '+StrPas(@Mmb.ObjectName), Err,     CFGErrorThreshold);
                                    if StrComp(Prop2,'GROUPS_I''M_IN')=0 then  {bug fix 95-10-31}
                                       cCode := NWAddObjectToSet(connID[dstSrv], @Mmb.ObjectName, Mmb.ObjectType,
                                                'SECURITY_EQUALS', @O2.ObjectName, O2.ObjectType);
                                    end;

                                 end;
                              end;
                          end;
                       end;
                       end; {case}
                    inc(PV.dataSetIndex);
                    until (cCodeV<>0) or (PV.moreFlag=0);
                    end;
                 end;
              until cCodeP<>0;
           end;
        end;
   SelectEntryFromMainList(SelectedListBox.ItemIndex+1, lastSrv);
   ConfigDialog.CopyOverwrite.Checked := OldSwitchOverwrite;
   Screen.Cursor := crDefault;
end;

procedure TMainScreen.CopyButtonClick(Sender: TObject);
begin
   CopyOrCloneObject(Sender, lastSrv, 1-lastSrv);
end;

procedure TMainScreen.CloneButtonClick(Sender: TObject);
begin
   CopyOrCloneObject(Sender, lastSrv, lastSrv);
end;

procedure TMainScreen.EditRenameChange(Sender: TObject);
VAR Pos:  Integer;
begin
   ItemSelected(Sender, Pos);
   RenButton.Enabled   := ((EditRename.Text<>'') and (Pos<>-1));
   CopyButton.Enabled  := ({(EditRename.Text<>'') and} (Pos<>-1));
   CloneButton.Enabled := ((EditRename.Text<>'') and (Pos<>-1));
   AddButton.Enabled   := ((EditRename.Text<>'') and (Pos<>-1));
   if ((EditRename.Text<>'') and (Pos<>-1)) then
      if lastSrv = 0 THEN CopyButton.Caption := '>> Copy &as >>'
                     else CopyButton.Caption := '<< Copy &as <<';
   DisableMenuItems;
end;

procedure TMainScreen.GetNameFromEditBox(Sender: TObject; VAR Name: String);
VAR
    NewNameCap:        TCaption;
    NewName:           String absolute NewNameCap;
    PName:             PChar;
BEGIN
    NewNameCap                  := EditRename.Text;
    Name                        := NewName;
    AddTermination(Name);
{    Name[Length(NewName)+1]     := #0; }
    PName                       := @Name[1];
    StrUpper(PName);
END;

procedure TMainScreen.RenameObject(Sender: TObject; VAR O: scanObjectRep; VAR NewName: String; SelectedListbox: TListBox);
{ NewName must be #0 terminated }
begin
        cCode := NWRenameObject(connID[lastSrv], StrUpper(@O.ObjectName[1]), @NewName[1], O.ObjectType);
        if (cCode=0) then
           begin
           ChangeCurrentListboxEntry(NewName, SelectedListbox);
           EditRename.Text := '';
           END
             else
           ShowError(cCode, 'NWRenameObject '+StrPas(@O.ObjectName), Err, CFGErrorThreshold);
end;

procedure TMainScreen.RenButtonClick(Sender: TObject);
VAR {Pos, NewPos:       Integer; }
    O:                 scanObjectRep;
    {SelectedListbox:   TListBox;}
    NewName:           String;
begin
   if lastSrv=0 THEN SelectedListbox    := ListBox0
                else SelectedListbox    := ListBox1;
   GetCurrentObject(Sender, SelectedListbox, O);
   IF ((cCode=0) and (EditRename.Text <> '')) THEN
        BEGIN
        GetNameFromEditBox(Sender, NewName);
        RenameObject(Sender, O, NewName, SelectedListbox);
        END;
end;

procedure TMainScreen.EditAddChange(Sender: TObject);
begin
   AddButton.Enabled := (EditRename.Text<>'');
   DisableMenuItems;
end;

procedure TMainScreen.EditDefaults1Click(Sender: TObject);
begin
   ConfigDialog.Show;
end;

procedure TMainScreen.ShowObjectDetails(Sender: TObject; PageName: String);
begin
    ShowButtonClick(Sender); { pretend 'Show' button was pressed }
    MultPageDlg.NoteBook_Info.ActivePage := PageName;
    MultPageDlg.TabSet.TabIndex     := MultPageDlg.Notebook_Info.PageIndex;
end;

procedure TMainScreen.ShowObjectClick(Sender: TObject);
begin
    ShowObjectDetails(Sender, 'Info');
end;

procedure TMainScreen.ShowTrustees1Click(Sender: TObject);
begin
    ShowObjectDetails(Sender, 'Trustees');
end;

procedure TMainScreen.ShowGroupsMembers1Click(Sender: TObject);
begin
    ShowObjectDetails(Sender, 'Group-Member');
end;

procedure TMainScreen.ShowSecurityEquivalences1Click(Sender: TObject);
begin
    ShowObjectDetails(Sender, 'Sec Equ');
end;

procedure TMainScreen.FormShow(Sender: TObject);
VAR x : Byte;
begin
   RadioGroupObjects.ItemIndex := 2;
   DefineSearchObject(Sender);
   EditRename.Text       := '';
   EditDefSearchObj.Text := CFGsearchPattern;
   CheckAttachments;
end;

procedure TMainScreen.EditDefSearchObjChange(Sender: TObject);
begin
  IF EditDefSearchObj.Text = '' THEN EditDefSearchObj.Text := '*';
end;

procedure TMainScreen.EditDefSearchObjExit(Sender: TObject);
begin
  EditDefSearchObj.Text := UpperCase(EditDefSearchObj.Text);
  IF CFGsearchPattern <> EditDefSearchObj.Text THEN begin
     CFGsearchPattern   := EditDefSearchObj.Text;
     AddTermination(CFGsearchPattern);
     ResetListClick(Sender);
     END;
end;

procedure TMainScreen.GetErrorList1Click(Sender: TObject);
begin
   XferHelp.FormErrList.Show;
end;

end.
