unit Xferobj;

interface

uses
  WinTypes, WinProcs, Classes, Graphics, Forms, Controls, StdCtrls,
  Tabs, SysUtils, Buttons, Dialogs, ExtCtrls,
  NWCalls,
  XferProp, XferUnit, XferAdd, Delphi_U;

type
  TMultPageDlg = class(TForm)
    ButtonPanel: TPanel;
    TabSet: TTabSet;
    Notebook_Info: TNotebook;
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    HelpBtn: TBitBtn;
    GroupBox0_Info: TGroupBox;
    GroupBox1_Trustees: TGroupBox;
    GroupBox2_GrpMmb: TGroupBox;
    ObjID: TLabel;
    OName: TEdit;
    OID: TEdit;
    ListBoxTrustee: TListBox;
    OType: TEdit;
    Label1: TLabel;
    ListBoxGrpMmb: TListBox;
    LabelMmbCount: TLabel;
    EditFullName: TEdit;
    LabelFullName: TLabel;
    GroupBox3_SecEqu: TGroupBox;
    ListBoxSecEqu: TListBox;
    LabelSecEqu: TLabel;
    LabelItem: TLabel;
    GroupBox4_Debug: TGroupBox;
    ListBoxDebug: TListBox;
    RadioGroupSetItem: TRadioGroup;
    RadioGroupWrite: TRadioGroup;
    RadioGroupRead: TRadioGroup;
    CheckBoxPropVal: TCheckBox;
    RadioGroupWrObj: TRadioGroup;
    RadioGroupRdObj: TRadioGroup;
    RadioGroupDynStat: TRadioGroup;
    RadioGroupPropFlag: TRadioGroup;
    ButtonDebug: TButton;
    ButtonDel: TButton;
    ButtonIns: TButton;

    procedure FormCreate(Sender: TObject);
    procedure FillTrusteeListBox(Sender: TObject);
    procedure FillGrpMmb(Sender: TObject);
    procedure FillSecEqu(Sender: TObject);
    procedure FillDebug(Sender: TObject);
    procedure TabSetClick(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
    procedure CancelBtnClick(Sender: TObject);
    procedure HelpBtnClick(Sender: TObject);
    procedure FillFirstPage(Sender: TObject; O: scanObjectRep; VAR LBox: TListBox);
    procedure UpdateListAndButton(Sender: TObject; Action: Byte; VAR TargetList: TListBox; VAR DelButton: TButton);
    procedure ButtonDelClick(Sender: TObject);
    procedure ButtonInsClick(Sender: TObject);
    procedure GetCurrentProperty(Sender: TObject; VAR Propname: String; VAR PropPos: Integer);
    procedure ListBoxDebugDblClick(Sender: TObject);
    procedure ListBoxDebugClick(Sender: TObject);
    procedure ButtonShowDebugClick(Sender: TObject);
  private
    { Private declarations }

  public
    { Public declarations }
  end;

var
  MultPageDlg   : TMultPageDlg;
  OldObjName    : TCaption;
  OldFullName   : TCaption;
  OldObjSec     : BYTE;
  MmbPos        : Integer;      { Position of Selected Item in Group Member Box }
  MmbType       : NWOBJ_TYPE;
  PropName      : String;
  PropPos       : Integer;
  CurrentListBox: TListBox;

implementation

{$R *.DFM}

procedure TMultPageDlg.FormCreate(Sender: TObject);

begin
   { make the number of Tabs = number of Pages, and have the same names }
   TabSet.Tabs := Notebook_Info.Pages;

   { make only some of the pages visible }

end;

procedure TMultPageDlg.FillFirstPage(Sender: TObject; O: scanObjectRep; VAR LBox: TListBox);
begin
   CurrentObject   := O;

   MultPageDlg.Close;

   MultPageDlg.OName.Text          := O.ObjectName;
   MultPageDlg.OID.Text            := hex8(O.objectID);
   MultPageDlg.OType.Text          := '0x'+hex4(Swap(O.objectType));

   RadioGroupWrObj.ItemIndex       := (O.ObjectSecurity DIV $10);
   RadioGroupRdObj.ItemIndex       := (O.ObjectSecurity MOD $10);
   RadioGroupDynStat.ItemIndex     := O.ObjectFlag;

   ButtonDel.Enabled := False;
   ButtonIns.Enabled := False;

   PV.dataSetIndex := 1;
   cCode := NWReadPropertyValue(connID[lastSrv], @O.ObjectName[1], O.ObjectType,  { get full name }
           'IDENTIFICATION', 1, PV.data, PV.moreFlag, PV.propertyFlags);
   if cCode=0 THEN MultPageDlg.EditFullName.Text := ASCIIz(PV.data)
              else MultPageDlg.EditFullName.Text := '';

   MultPageDlg.Show;
   Notebook_Info.ActivePage := 'Info';
   TabSet.TabIndex     := Notebook_Info.PageIndex;

   OldObjName          := MultPageDlg.OName.Text;
   OldFullName         := MultPageDlg.EditFullName.Text;
   OldObjSec           := O.ObjectSecurity;

   case Swap(O.ObjectType) of
        OT_User_Group: TabSet.Tabs.Strings[2] := 'Members';
        OT_User:       TabSet.Tabs.Strings[2] := 'Groups';
        end;

   GroupBox2_GrpMmb.Caption := TabSet.Tabs.Strings[2];
end;

procedure TMultPageDlg.FillTrusteeListBox(Sender: TObject);
VAR
  volumeNumber:                NWVOL_NUM;
  sequenceNumber:              NWTRUSTEE_SEQUENCE_NUM;
  trusteeAccessRights_B:       NWACCESS_RIGHTS;
  trusteeAccessRights_W:       NWDIR_TRUSTEE_RIGHTS;
  path:                        PChar;
  NWRights:                    Str8;

begin
    Screen.Cursor := crHourGlass;
{    NWVers := NW_Version(connID[lastSrv]); }
    path := StrAlloc(255);
    ClearList(ListBoxTrustee);
    For volumeNumber   := 0 to 31 DO Begin
        sequenceNumber := NWTRUSTEE_SEQUENCE_NUM(-1);
        repeat
           cCode := NWScanObjectTrusteePaths(connID[lastSrv], usrInfo[lastSrv].ObjectID,
                    volumeNumber, sequenceNumber, trusteeAccessRights_B, path);
           if ( (path[1]=#0) or (sequenceNumber=0)) THEN cCode := $FFFF;
           if cCode=0 THEN BEGIN
              trusteeAccessRights_W := trusteeAccessRights_B;
              NWRights := Rights(trusteeAccessRights_W, NWVers[lastSrv]);
              ListBoxTrustee.Items.Add('['+NWRights+'] '+StrPas(path)+#0);
              END;
           until (cCode<>0);
        END;
    StrDispose(path);
    Screen.Cursor := crDefault;
end;

procedure AddPropertiesToList(Sender: TObject; VAR TargetList: TListBox; OName: PChar; OType: NWOBJ_TYPE);
VAR
    Prop:              scanPropRep;
begin
    Screen.Cursor := crHourGlass;
    ClearList(TargetList);
    Prop.sequenceNumber := -1;
    repeat
       cCode := NWScanProperty(connID[lastSrv], OName, OType, '*',
                Prop.sequenceNumber, Prop.propertyName, Prop.propertyFlags,
                Prop.propertySecurity, Prop.hasValueFlag, Prop.moreFlag);
                if (cCode<>$89FB) THEN ShowError(cCode, 'NWScanProperty '+Prop.propertyName, Err, CFGErrorThreshold);
       if cCode=0 then TargetList.Items.Add(Prop.propertyName);
       until cCode<>0;
    Screen.Cursor := crDefault;
end;


procedure TMultPageDlg.FillGrpMmb(Sender: TObject);
begin
    case Swap(usrInfo[lastSrv].ObjectType) of
         OT_USER:        propName := 'GROUPS_I''M_IN';
         OT_USER_GROUP:  propName := 'GROUP_MEMBERS';
         else            propName := '';
         end; {case}
    AddTermination(propName);
    AddSetToList(Sender, ListBoxGrpMmb, @usrInfo[lastSrv].ObjectName, usrInfo[lastSrv].ObjectType, @propName[1]);
    case Swap(usrInfo[lastSrv].ObjectType) of
         OT_USER:        LabelMmbCount.Caption := IntToStr(ListBoxGrpMmb.Items.Count) + ' Groups';
         OT_USER_GROUP:  LabelMmbCount.Caption := IntToStr(ListBoxGrpMmb.Items.Count) + ' Members';
         else            LabelMmbCount.Caption := 'Data not Collected';
         end; {case}

   SelectDefaultListItem(Sender, ListBoxGrpMmb, MmbPos);

   if Swap(CurrentObject.ObjectType) = OT_USER
      THEN MmbType  := Swap(OT_USER_GROUP)
      else MmbType  := Swap(OT_USER);

   ButtonDel.Enabled := (MmbPos>=0);
end;

procedure TMultPageDlg.FillSecEqu(Sender: TObject);
begin
   propName := 'SECURITY_EQUALS';   AddTermination(propName);
   AddSetToList(Sender, ListBoxSecEqu, @usrInfo[lastSrv].ObjectName, usrInfo[lastSrv].ObjectType, @propName[1]);
   LabelSecEqu.Caption := IntToStr(ListBoxSecEqu.Items.Count) + ' Security Equivalences';
   SelectDefaultListItem(Sender, ListBoxSecEqu, MmbPos);
   MmbType  := Swap(OT_USER);
   ButtonDel.Enabled := (MmbPos>=0);
end;

procedure TMultPageDlg.FillDebug(Sender: TObject);
begin
   AddPropertiesToList(Sender, ListBoxDebug, @usrInfo[lastSrv].ObjectName, usrInfo[lastSrv].ObjectType);
   GetCurrentProperty(Sender, PropName, PropPos);
   ButtonDel.Enabled := (PropPos>=0);
end;

procedure TMultPageDlg.TabSetClick(Sender: TObject);
begin
  { set visible Page to which Tab was clicked }
  Notebook_Info.PageIndex := TabSet.TabIndex;
  ButtonDel.Enabled := False;
  ButtonIns.Enabled := False;
  CancelBtn.Enabled := (TabSet.TabIndex=0);  {only page 0 may be canceled}
  case TabSet.TabIndex of
       1: Begin
          CurrentListBox := ListBoxTrustee;
          FillTrusteeListBox(Sender);
          end;
       2: Begin
          CurrentListBox := ListBoxGrpMmb;
          ButtonIns.Enabled := True;
          FillGrpMmb(Sender);
          end;
       3: Begin
          CurrentListBox := ListBoxSecEqu;
          ButtonIns.Enabled := True;
          FillSecEqu(Sender);
          end;
       4: Begin
          CurrentListBox := ListBoxDebug;
          FillDebug(Sender);
          end;
       end; { case }
end;

procedure TMultPageDlg.OKBtnClick(Sender: TObject);
VAR NewEntry:     String;
    NewObjSec:    BYTE;
begin
   if (OldFullName <> MultPageDlg.EditFullName.Text) THEN BEGIN   { full name has changed }
      NewEntry := MultPageDlg.EditFullName.Text + #0;
      cCode := WriteFullName(connID[lastSrv], @CurrentObject.ObjectName, CurrentObject.ObjectType, NewEntry[1]);
      if (cCode>0) then ShowError(cCode, 'NWChangeFullName '+StrPas(@CurrentObject.ObjectName), Err, CFGErrorThreshold);
      end;

   IF (OldObjName <> MultPageDlg.OName.Text) THEN BEGIN           { obj name has changed }
      NewEntry := UpperString(MultPageDlg.OName.Text); if Length(NewEntry)>48 then NewEntry[0] := #48;
      IF (NewEntry <> '') THEN
         BEGIN
         AddTermination(NewEntry);
         cCode := NWRenameObject(connID[lastSrv], @CurrentObject.ObjectName[1], @NewEntry[1], CurrentObject.ObjectType);
         if (cCode=0) then begin
                           ChangeCurrentListboxEntry(NewEntry, SelectedListbox);
                           Move(NewEntry[1], CurrentObject.ObjectName, Length(NewEntry));
                           end
                      else ShowError(cCode, 'NWRenameObject '+StrPas(@CurrentObject.ObjectName), Err, CFGErrorThreshold);
         END;
      END;

   NewObjSec := MultPageDlg.RadioGroupWrObj.ItemIndex * $10 + MultPageDlg.RadioGroupRdObj.ItemIndex;
   if (OldObjSec <> NewObjSec) THEN BEGIN   { obj security has changed }
      cCode := NWChangeObjectSecurity(connID[lastSrv], @CurrentObject.ObjectName[1], CurrentObject.ObjectType, NewObjSec);
      if (cCode>0) then ShowError(cCode, 'NWChangeObjectSec '+StrPas(@CurrentObject.ObjectName), Err, CFGErrorThreshold);
      END;

   FormProp.Close;
   MultPageDlg.Close;
end;

procedure TMultPageDlg.CancelBtnClick(Sender: TObject);
begin
   FormProp.Close;
   MultPageDlg.Close;
end;

procedure TMultPageDlg.HelpBtnClick(Sender: TObject);
begin
   NotYet(Sender);
end;

procedure TMultPageDlg.UpdateListAndButton(Sender: TObject; Action: Byte;
          VAR TargetList: TListBox; VAR DelButton: TButton);
VAR
    MmbName : String;
begin
   if ( (Action = 0) and (TargetList.ItemIndex>=0) ) then begin  { 'del' action }
      MmbName := TargetList.Items[TargetList.ItemIndex];
      MmbPos  := TargetList.ItemIndex;
      AddTermination(MmbName);
      cCode := DelAndSyncMembership(connID[lastSrv], @CurrentObject.ObjectName[1], CurrentObject.ObjectType,
                  @PropName[1], @MmbName[1], MmbType);  { sync Group/Members/SecEqu if req'd }
      ShowError(cCode, 'NWDeleteObjectFromSet ', Err, CFGErrorThreshold);
      if cCode=0 then DeleteListEntry(Sender, TargetList, MmbPos);
      IF TargetList.Items.Count <= 0 THEN DelButton.Enabled := FALSE;
      end
       else
   if Action = 1 then begin  { 'add' action }
      AddObjectDialog.ListBoxCreate(Sender, connID[lastSrv], '*', MmbType);
      if chosenObj <> '' then begin { was a name selected ?? }
         cCode := AddAndSyncMembership(connID[lastSrv], @CurrentObject.ObjectName[1], CurrentObject.ObjectType,
                  @PropName[1], @chosenObj[1], MmbType);  { sync Group/Members/SecEqu if req'd }
         ShowError(cCode, 'NWAddObjectToSet '+chosenObj+' ', Err, CFGErrorThreshold);
         if (cCode = 0) then begin
            AddListEntry(Sender, TargetList, chosenObj);
            DelButton.Enabled := TRUE;
            end
         end;
      end
end;

procedure TMultPageDlg.ButtonDelClick(Sender: TObject);
VAR
    MmbName : String;
begin
  SelectDefaultListItem(Sender, CurrentListBox, MmbPos);
  case TabSet.TabIndex of
       2,3: UpdateListAndButton(Sender, 0, CurrentListBox, ButtonDel);
       4: Begin
          if (CurrentListBox.ItemIndex>=0) then begin  { 'del' action }
             MmbName := CurrentListBox.Items[CurrentListBox.ItemIndex];
             MmbPos  := CurrentListBox.ItemIndex;
             AddTermination(MmbName);
             cCode := NWDeleteProperty(connID[lastSrv], @CurrentObject.ObjectName[1], CurrentObject.ObjectType, @PropName[1]);
             ShowError(cCode, 'NWDeleteProperty '+PropName+' ', Err, CFGErrorThreshold);
             if cCode=0 then BEGIN
                DeleteListEntry(Sender, CurrentListBox, MmbPos);
                ButtonDel.Enabled := CurrentListBox.Items.Count >= 0;
                end;
             end
          end;
       end; { case }
end;

procedure TMultPageDlg.ButtonInsClick(Sender: TObject);
begin
   UpdateListAndButton(Sender, 1, CurrentListBox, ButtonDel);
end;

procedure TMultPageDlg.GetCurrentProperty(Sender: TObject; VAR Propname: String; VAR PropPos: Integer);
begin
   PropName := '';
   PropPos   := 0;
   SelectDefaultListItem(Sender, ListBoxDebug, PropPos);
   IF (PropPos >= 0) THEN begin
      PropName := ListBoxDebug.Items[ListBoxDebug.ItemIndex];
      AddTermination(PropName);
      CurrentProperty.sequenceNumber := -1;
      cCode := NWScanProperty(connID[lastSrv], @CurrentObject.ObjectName[1], CurrentObject.ObjectType, @PropName[1],
                CurrentProperty.sequenceNumber, CurrentProperty.propertyName, CurrentProperty.propertyFlags,
                CurrentProperty.propertySecurity, CurrentProperty.hasValueFlag, CurrentProperty.moreFlag);
                if (cCode<>$89FB) THEN ShowError(cCode, 'NWScanProperty '+PropName, Err, CFGErrorThreshold);
      if (cCode = 0) then begin
          RadioGroupSetItem.ItemIndex := (CurrentProperty.propertyFlags    and BF_SET);
          RadioGroupPropFlag.ItemIndex:= (CurrentProperty.propertyFlags    and BF_DYNAMIC);
          RadioGroupWrite.ItemIndex   := (CurrentProperty.propertySecurity div $10);
          RadioGroupRead.ItemIndex    := (CurrentProperty.propertySecurity Mod $10);
          CheckBoxPropVal.Checked     := (CurrentProperty.hasValueFlag <> 0);
          ButtonDebug.Enabled         := CheckBoxPropVal.Checked;
          end;
      END;
   FormProp.Close;
end;

procedure TMultPageDlg.ListBoxDebugDblClick(Sender: TObject);
begin
   GetCurrentProperty(Sender, Propname, PropPos);
   if ( (PropPos >= 0) and (CurrentProperty.hasValueFlag <> 0) ) then begin
      PV.dataSetIndex := 1;
      cCode := NWReadPropertyValue(connID[lastSrv], @CurrentObject.ObjectName[1], CurrentObject.ObjectType,
               @CurrentProperty.propertyName[1], 1, PV.data, PV.moreFlag, PV.propertyFlags);
      if cCode=0 then begin
         FormProp.FormCreate(Sender);
         FormProp.ShowModal;
         end;
      end;
end;

procedure TMultPageDlg.ListBoxDebugClick(Sender: TObject);
begin  { called when one prop is selected }
   GetCurrentProperty(Sender, PropName, PropPos);
end;

procedure TMultPageDlg.ButtonShowDebugClick(Sender: TObject);
begin
   ListBoxDebugDblClick(Sender);
end;

end.
