unit XferOpt;

interface

uses NWCalls, XferUnit, Delphi_U,
     WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
     IniFiles, StdCtrls, SysUtils, ExtCtrls;

type
  TConfigDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    HelpBtn: TBitBtn;
    Bevel1: TBevel;
    GroupBoxConfig: TGroupBox;
    CopyOverwrite: TCheckBox;
    CopyPartial: TCheckBox;
    CopyTrustees: TCheckBox;
    CheckWarnDel: TCheckBox;
    WarningLevel: TRadioGroup;
    CheckAutoSave: TCheckBox;
    ButtonSaveOptions: TButton;
    ButtonLoadOptions: TButton;
    EditDefSearchObj: TEdit;
    LabelDefSearchObj: TLabel;
    RadioGroupDefPWD: TRadioGroup;
    EditDefPWD: TEdit;
    procedure OKBtnClick(Sender: TObject);
    procedure CancelBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure LoadOptions(Sender: TObject);
    procedure ButtonSaveOptionsClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure EditDefSearchObjChange(Sender: TObject);
    procedure GetGlobalCFG(Sender: TObject);
    procedure RadioGroupDefPWDClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  ConfigDialog: TConfigDialog;

implementation

{$R *.DFM}


procedure TConfigDialog.OKBtnClick(Sender: TObject);
begin
   ConfigDialog.Close;
end;

procedure TConfigDialog.CancelBtnClick(Sender: TObject);
begin
   ConfigDialog.Close;
end;

procedure TConfigDialog.FormCreate(Sender: TObject);
begin
   LoadOptions(Sender);
   GetGlobalCFG(Sender);
end;

procedure TConfigDialog.LoadOptions(Sender: TObject);
var
  XFerIni: TIniFile;
begin
  XFerIni := TIniFile.Create('XFER.INI');
  with XferIni do
  begin
    CheckAutoSave.Checked      := ReadBool('Options', 'AutoSave',            TRUE);
    CheckWarnDel.Checked       := ReadBool('Options', 'ConfirmDeletions',    FALSE);
    WarningLevel.ItemIndex     := ReadInteger('Options', 'WarningLevel',     Pred(CFGErrorThreshold));
    CopyOverwrite.Checked      := ReadBool('Copy Options', 'OverwriteProperties', TRUE);
    CopyTrustees.Checked       := ReadBool('Copy Options', 'CopyTrustees',        TRUE);
    CopyPartial.Checked        := ReadBool('Copy Options', 'CopyPartial',         TRUE);
    RadioGroupDefPWD.ItemIndex := ReadInteger('Password Options', 'DefaultPassword',  1);
    EditDefPWD.Text            := ReadString( 'Password Options', 'CustomPassword',   '');
    EditDefSearchObj.Text      := ReadString( 'Search Options',   'SearchPattern',    '*');
    Free;
    end
end;

procedure TConfigDialog.ButtonSaveOptionsClick(Sender: TObject);
var
  XFerIni: TIniFile;
begin
  XFerIni := TIniFile.Create('XFER.INI');
  with XferIni do
  begin
    WriteBool('Options', 'AutoSave',            CheckAutoSave.Checked);
    WriteBool('Options', 'ConfirmDeletions',    CheckWarnDel.Checked);
    WriteInteger('Options', 'WarningLevel',     WarningLevel.ItemIndex);
    WriteBool('Copy Options', 'OverwriteProperties', CopyOverwrite.Checked);
    WriteBool('Copy Options', 'CopyTrustees',        CopyTrustees.Checked);
    WriteBool('Copy Options', 'CopyPartial',         CopyPartial.Checked);
    WriteInteger('Password Options', 'DefaultPassword',  RadioGroupDefPWD.ItemIndex);
    WriteString( 'Password Options', 'CustomPassword',   EditDefPWD.Text);
    WriteString( 'Search Options',   'SearchPattern',    EditDefSearchObj.Text);
    Free;
    end
end;

procedure TConfigDialog.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  GetGlobalCFG(Sender);
  If CheckAutoSave.Checked THEN ButtonSaveOptionsClick(Sender);
end;

procedure TConfigDialog.EditDefSearchObjChange(Sender: TObject);
begin
   IF EditDefSearchObj.Text = '' THEN EditDefSearchObj.Text := '*';
end;

procedure TConfigDialog.GetGlobalCFG(Sender: TObject);
begin
  EditDefPWD.Text       := UpperCase(EditDefPWD.Text);
  Case RadioGroupDefPWD.ItemIndex of
    0: CFGDefPWD := '&username';
    1: CFGDefPWD := '';
    2: CFGDefPWD := EditDefPWD.Text;
    3: CFGDefPWD := '&ask';
    end; {case}
  AddTermination(CFGDefPWD);

  EditDefSearchObj.Text := UpperCase(EditDefSearchObj.Text);
  CFGsearchPattern      := EditDefSearchObj.Text;
  AddTermination(CFGSearchPattern);

  CFGErrorThreshold     := Succ(WarningLevel.ItemIndex);
end;

procedure TConfigDialog.RadioGroupDefPWDClick(Sender: TObject);
begin
   if RadioGroupDefPWD.ItemIndex = 3 THEN begin
      NotYet(Sender);
      RadioGroupDefPWD.ItemIndex := 1; 
      end;
end;

end.
