unit XferProp;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Dialogs, Graphics, Controls,
  StdCtrls, ExtCtrls, Forms, Tabs, Buttons, Grids,
  NWCalls, XferUnit, Delphi_U
  ;

type
  TFormProp = class(TForm)
    ButtonDebOK: TBitBtn;
    ButtonDebCancel: TBitBtn;
    ButtonDebHelp: TBitBtn;
    TabSetDebug: TTabSet;
    Notebook_Debug: TNotebook;
    GroupBoxDebHex: TGroupBox;
    GridHex: TStringGrid;
    GroupBoxDebSet: TGroupBox;
    ListBoxDebSet: TListBox;
    LabelHexInt: TLabel;
    LabelLong2: TLabel;
    LabelHexOffset: TLabel;
    LabelOffset: TLabel;
    LabelInt: TLabel;
    LabelLong: TLabel;
    EditHex: TEdit;
    CheckBoxChanged: TCheckBox;
    ButtonHexSetDel: TButton;
    ButtonHexSetIns: TButton;
    LabelChar: TLabel;
    LabelHexChar: TLabel;
    procedure Changes(Sender: TObject; Chg: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure TabClick(Sender: TObject);
    procedure SetGridData(Sender: TObject; Offset: BYTE; NewVal: BYTE);
    procedure ButtonDebOKClick(Sender: TObject);
    procedure ButtonDebCancelClick(Sender: TObject);
    procedure ButtonDebHelpClick(Sender: TObject);
    procedure GridHexClick(Sender: TObject);
    procedure GridHexSetEditText(Sender: TObject; ACol, ARow: Longint; const Value: String);
    procedure GridHexGetEditText(Sender: TObject; ACol, ARow: Longint;
      var Value: OpenString);
    procedure CheckBoxChangedClick(Sender: TObject);
    procedure ButtonHexSetDelClick(Sender: TObject);
  end;

var
  FormProp    : TFormProp;
  ChangesDone : Boolean;

implementation

{$R *.DFM}

procedure TFormProp.Changes(Sender: TObject; Chg: Boolean);
begin
   ChangesDone := Chg;
   CheckBoxChanged.Checked := Chg;
end;

procedure GetStrFromData(Var sTemp: String);
Var i : BYTE;
begin
   sTemp := '';
   for i := 1 to 128 do
       case PV.data[i] of
            0:      break;
            1..31:  sTemp := sTemp + '.';
            else    sTemp := sTemp + chr(PV.data[i]);
            end; {case}
end;

procedure TFormProp.FormCreate(Sender: TObject);
Var sTemp : String;
begin
   GetStrFromData(sTemp); EditHex.Text := sTemp;
   TabSetDebug.Tabs     := Notebook_Debug.Pages;
{   TabSetDebug.TabIndex := Notebook_Debug.PageIndex; }
   if (CurrentProperty.propertyFlags and BF_SET)>0
      then Notebook_Debug.ActivePage := 'Set'
      else Notebook_Debug.ActivePage := 'Hex';
   TabSetDebug.TabIndex := Notebook_Debug.PageIndex;
{   FormProp.Show; }
end;

procedure TFormProp.SetGridData(Sender: TObject; Offset: BYTE; NewVal: BYTE);
begin
   GridHex.Cells[Succ((Offset-1) MOD (GridHex.ColCount-1)), Succ((Offset-1) DIV (GridHex.ColCount-1))] := hex2(NewVal);
   PV.data[Offset] := NewVal;
end;

procedure AnalyseSet(Sender: TObject; VAR TargetList: TListBox);
VAR
  cCodeV:            NWCCODE;
  i:                 byte;
  SetArray:          Array[0..31] of NWOBJ_ID;
  Mmb:               TBinderyObject;
begin
    Screen.Cursor := crHourGlass;
    ClearList(TargetList);
    Move(PV.data, SetArray, SizeOf(SetArray));
    for i := 0 to 31 do
        if SetArray[i]<>0 THEN begin { Read set members }
           cCode := NWGetObjectName(connID[lastSrv], SetArray[i], Mmb.ObjectName, Mmb.ObjectType);
           ShowError(cCode, 'NWGetMmbName '+hex8(SetArray[i]), Err, CFGErrorThreshold);
           if cCode=0 then TargetList.Items.Add(Mmb.ObjectName);
           end;
    Screen.Cursor := crDefault;
end;



procedure TFormProp.TabClick(Sender: TObject);
VAR i,j    : Byte;
    sTemp  : String;
    xCol, xRow: BYTE;
begin
  Notebook_Debug.PageIndex := TabSetDebug.TabIndex;
  if Notebook_Debug.ActivePage = 'Hex' then begin
     Changes(Sender, False);
     xCol := GridHex.ColCount-1;
     xRow := GridHex.RowCount-1;
     for i := 1 to xCol do begin
         sTemp := hex2(i-1);  sTemp[1] := 'x';
         GridHex.Cells[i, 0] := sTemp;
         end;
     for i := 1 to xRow do begin
         sTemp := hex2((i-1)*xCol);  sTemp[2] := 'x';
         GridHex.Cells[0, i] := sTemp;
         end;
     for i := 1 to 128  do SetGridData(Sender, i, PV.data[i]);
     GroupBoxDebHex.Caption := 'Hex Representation of '+CurrentProperty.propertyName;
     end
      else
  if Notebook_Debug.ActivePage = 'Set' then begin
     if (CurrentProperty.propertyFlags and BF_SET)>0 then AnalyseSet(Sender, ListBoxDebSet);
     end;
end;

procedure TFormProp.ButtonDebOKClick(Sender: TObject);
begin
   if ChangesDone then begin
      cCode := NWWritePropertyValue(connID[lastSrv], @CurrentObject.ObjectName[1], CurrentObject.ObjectType,
               @CurrentProperty.propertyName[1], 1, PV.data, PV.moreFlag);
      ShowError(cCode, 'NWWritePropertyValue '+CurrentProperty.propertyName, Err, CFGErrorThreshold);
      end;
   FormProp.Close;
end;

procedure TFormProp.ButtonDebCancelClick(Sender: TObject);
begin
   FormProp.Close;
end;

procedure TFormProp.ButtonDebHelpClick(Sender: TObject);
begin
   NotYet(Sender);
end;

procedure TFormProp.GridHexClick(Sender: TObject);
Var offs, data : Integer;
    LTemp      : LongInt;
    WTemp      : Word absolute LTemp;
    BTemp      : Byte absolute LTemp;
    CTemp      : Char absolute LTemp;
begin
   offs := GridHex.Col + Pred(GridHex.Row) * (GridHex.ColCount-1);
   move(PV.data[offs], LTemp, 4);
   data := BTemp;
   LabelHexOffset.Caption := IntToStr(offs) + ' ('+IntToStr(GridHex.Col)+'/'+IntToStr(GridHex.Row)+')';
   if (offs>=128) then LabelHexInt.Caption    := IntToStr(data)
                  else LabelHexInt.Caption    := IntToStr(data) + '/' + IntToStr(WTemp);
   if (offs>=126) then LabelLong2.Caption    := 'undef'
                  else LabelLong2.Caption    := IntToStr(LTemp);
   LabelHexChar.Caption   := CTemp; { Chr(data); }
end;

procedure TFormProp.GridHexSetEditText(Sender: TObject; ACol,
  ARow: Longint; const Value: String);
VAR i, offs : Integer;
    NewVal  : Str8;
    sTemp   : String;
    pTemp   : PChar;
begin
   offs := GridHex.Col + Pred(GridHex.Row) * (GridHex.ColCount-1);
   NewVal := Value; if Length(Value)>2 then NewVal := copy(Value, 2, 2);
   Val('$'+NewVal, i, cCode);
   GridHex.Cells[ACol, ARow] := NewVal;
   if (cCode=0) then begin
      PV.data[offs] := i;
      GetStrFromData(sTemp);
      EditHex.Text := sTemp;
      Changes(Sender, True);
      end
       else SetGridData(Sender, offs, PV.data[offs]);
end;

procedure TFormProp.GridHexGetEditText(Sender: TObject; ACol,
  ARow: Longint; var Value: OpenString);
begin
   LabelHexInt.Caption    := 'GE '+ Value + '('+IntToStr(ACol)+'/'+IntToStr(ARow)+')';
end;

procedure TFormProp.CheckBoxChangedClick(Sender: TObject);
begin
   ChangesDone := CheckBoxChanged.Checked;
end;

procedure TFormProp.ButtonHexSetDelClick(Sender: TObject);
VAR
  SetArray:    Array[0..31] of NWOBJ_ID;
  Mmb:         TBinderyObject;
  Pos:         Integer;
begin
   if ListBoxDebSet.ItemIndex <> -1 THEN begin
      Pos := ListBoxDebSet.ItemIndex;
      Move(PV.data, SetArray, SizeOf(SetArray));
      cCode := NWGetObjectName(connID[lastSrv], SetArray[Pos], Mmb.ObjectName, Mmb.ObjectType);
      ShowError(cCode, 'NWGetObjectName '+hex8(SetArray[Pos]), Err, CFGErrorThreshold);

      DelAndSyncMembership(connID[lastSrv], @CurrentObject.ObjectName[1], CurrentObject.ObjectType,
                                            @CurrentProperty.propertyName[1], @Mmb.ObjectName[1], Mmb.ObjectType);
      if cCode=0 then begin
         DeleteListEntry(Sender, ListBoxDebSet, Pos);
         SetArray[Pos] := 0;
         Move(SetArray, PV.data, SizeOf(SetArray));
         IF ListBoxDebSet.Items.Count <= 0 THEN ButtonHexSetDel.Enabled := FALSE;
         end;
      end;
end;

end.
